// C++ Implementation: addPackage
//
// Description: implementation of the add package dialog
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "addpackage.h"
#include "addpackage.moc"

#include <klocale.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <klineedit.h>
#include <ktextedit.h>

using namespace nbs;
using namespace dialog;

AddPackage::AddPackage(QWidget* parent, const char* name)
	: KDialogBase(parent, name, true, i18n("Add Package"), Ok | Cancel) {

	QFrame *topFrame = makeMainWidget();
	QGridLayout *topLayout = new QGridLayout(topFrame);
	topLayout->setSpacing(spacingHint());

	QLabel *label = new QLabel(i18n("Name:"), topFrame);
	topLayout->addWidget(label, 0, 0);
	lineEditName = new KLineEdit(topFrame);
	topLayout->addWidget(lineEditName, 0, 1);
	connect(lineEditName, SIGNAL(textChanged(const QString&)), SLOT(enableAddButton()));
	lineEditName->setFocus();

	label = new QLabel(i18n("Title:"), topFrame);
	topLayout->addWidget(label, 1, 0);
	lineEditTitle = new KLineEdit(topFrame);
	topLayout->addWidget(lineEditTitle, 1, 1);
	connect(lineEditTitle, SIGNAL(textChanged(const QString&)), SLOT(enableAddButton()));

	label = new QLabel(i18n("Description:"), topFrame);
	topLayout->addMultiCellWidget(label, 2, 2, 0, 1);
	textEditDescription = new KTextEdit(topFrame);
	topLayout->addMultiCellWidget(textEditDescription, 3, 3, 0, 1);

	checkboxSpec = new QCheckBox(i18n("Create spec"), topFrame);
	topLayout->addMultiCellWidget(checkboxSpec, 4, 4, 0, 1);

	enableAddButton();
}

AddPackage::~AddPackage(){
}

void AddPackage::enableAddButton()
{
	enableButton(Ok, !lineEditName->text().isEmpty() && !lineEditTitle->text().isEmpty());
}

QString AddPackage::packageName() const
{
	return lineEditName->text();
}

QString AddPackage::packageTitle() const
{
	return lineEditTitle->text();
}

QString AddPackage::packageDescription() const
{
	return textEditDescription->text();
}

bool AddPackage::packageSpec() const
{
	return false;
}
