/*
    This file is part of Richclient.

    Copyright (c) 2007 SUSE LINUX Products GmbH

    Author: Cornelius Schumacher <cschum@suse.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "httpheader.h"

#include <kio/jobclasses.h>
#include <kdebug.h>

#include <qstringlist.h>

HttpHeader::HttpHeader()
{
}

HttpHeader::HttpHeader( KIO::Job *job )
{
	parseMetaData( job );
}

void HttpHeader::parseMetaData( KIO::Job *job )
{
	mFields.clear();
	mReturnCode = QString::null;
	mReturnText = QString::null;

	QString header = job->queryMetaData( "HTTP-Headers" );

//	kdDebug() << "HEADER: " << header << endl;

	if ( !header.isEmpty() ) {
		QStringList headerLines = QStringList::split( "\n", header );
		QStringList::ConstIterator it;
		for( it = headerLines.begin(); it != headerLines.end(); ++it ) {
			QString line = *it;
			if ( it == headerLines.begin() ) {
				int pos1 = line.find( " " );
				int pos2 = line.find( " ", pos1 + 1 );
				if ( pos1 < 0 || pos2 < 0 ) {
					kdError() << "Unable to parse first line of HTTP Header" << endl;
				} else {
					mReturnCode = line.mid( pos1 + 1, pos2 - pos1 - 1 );
//					kdDebug() << "RETURN CODE: '" << mReturnCode << "'" << endl;
					mReturnText = line.mid( pos2 + 1 );
//					kdDebug() << "RETURN TEXT: '" << mReturnText << "'" << endl;
				}
			} else {
				int p1 = line.findRev( ":" );
				if ( p1 > 0 ) {
					QString key = line.left( p1 );
					QString value = line.mid( p1 + 1 ).stripWhiteSpace();
//					kdDebug() << "HEADER KEY: '" << key << "' VALUE: '" << value << "'" << endl;
					mFields.insert( key, value );
				}
			}
		}
	}
}

QString HttpHeader::returnCode() const
{
	return mReturnCode;
}

QString HttpHeader::returnText() const
{
	return mReturnText;
}

bool HttpHeader::isError() const
{
	return !mReturnCode.isEmpty() && mReturnCode != "200";
}

QString HttpHeader::field( const QString &fieldName ) const
{
	return mFields[fieldName];
}
