// C++ Implementation: IMainDlg
//
// Description: Main widget implementation
//
// Main class containing ui function
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "imaindlg.h"
#include "dialog.h"

#include <kdebug.h>
#include <klocale.h>
#include <kaction.h>
#include <kurlrequester.h>
#include <kfiledialog.h>
#include <klistviewsearchline.h>

#include <qwidgetstack.h>

using namespace nbs;

IMainDlg::IMainDlg(QWidget* parent = 0,  const char* name = "IMainDlg") : MainViewBase(parent, name) {
	kdDebug() << "new IMainDlg" << endl;
	workset   = new MainWorker();
	kLineEditUserName->setText(workset->getUserName());

	stateMngr = new StateManager(this, "State Manager", workset);
	stateMngr->setState(st_none);

	setupButtons();
        setupConnections();
	setupListViews();
	setupTabs();

	if (!kLineEditUserName->text().isEmpty()) {
		stateMngr->setState(st_connect);}
}

IMainDlg::~IMainDlg() {
	delete workset;
	delete stateMngr;
        kdDebug() << "deleted IMainDlg" << endl;
}

void IMainDlg::setupButtons() {
	buttonSaveProject->setEnabled(false);
	buttonBuildProject->setEnabled(false);
	buttonAddPackage->setEnabled(false);
	buttonRemovePackage->setEnabled(false);
	buttonLinkPackage->setEnabled(false);
	buttonAddUserProject->setEnabled(false);
	buttonRemoveUserProject->setEnabled(false);

	kTextEditProjectTitle->setReadOnly(false);
	kTextEditProjectDescription->setReadOnly(false);

	buttonAddFile->setEnabled(false);
	buttonRemoveFile->setEnabled(false);
	buttonBuildPackage->setEnabled(false);
	buttonSavePackage->setEnabled(false);
	buttonAddUserPackage->setEnabled(false);
	buttonRemoveUserPackage->setEnabled(false);

	kTextEditPackageTitle->setReadOnly(false);
	kTextEditPackageDescription->setReadOnly(false);
}

void IMainDlg::setupListViews() {
	kListViewWatched->addColumn(tr("Watched Project"));
	kListViewWatched->setFullWidth(1);

	kListViewProject->addColumn(tr("Project"));
	kListViewProject->setFullWidth(1);

	kListViewPackage->addColumn(tr("Package"));
	kListViewPackage->setFullWidth(1);

	kListViewRepository->addColumn(tr("Repository"));
	kListViewRepository->setFullWidth(1);

	kListViewFiles->addColumn(tr("File"));
	kListViewFiles->addColumn(tr("Size"));
	kListViewFiles->setFullWidth(1);

	kListViewUser_project->addColumn(tr("Name"));
	kListViewUser_project->addColumn(tr("Role"));
	kListViewUser_project->addColumn(tr("Email"));
	kListViewUser_project->setFullWidth(1);

	kListViewUser_package->addColumn(tr("Name"));
	kListViewUser_package->addColumn(tr("Role"));
	kListViewUser_package->addColumn(tr("Email"));
	kListViewUser_package->setFullWidth(1);

	kListViewSearchLineProject->setListView(kListViewProject);
	kListViewSearchLinePackage->setListView(kListViewPackage);

}

void IMainDlg::setupConnections() {
	connect(kLineEditUserName, SIGNAL(textChanged(const QString&)), this, SLOT(changeUser(const QString&)));
	connect(kLineEditUserName, SIGNAL(returnPressed()), this, SLOT(buttonConnect_clicked()));

	connect(tabWidgetMain, SIGNAL(currentChanged(QWidget*)), this, SLOT(tabWidgetMain_currentChanged(QWidget*)));

	connect(kURLRequester, SIGNAL(openFileDialog(KURLRequester*)), this, SLOT(kURLRequester_openFileDialog(KURLRequester*)));
	connect(kURLRequester, SIGNAL(returnPressed(const QString&)), this, SLOT(kURLRequester_returnPressed(const QString&)));

	connect(comboBoxPackage, SIGNAL(activated(int)), this, SLOT(comboBoxPackage_activated(int)));
	connect(comboBoxProject, SIGNAL(activated(int)), this, SLOT(comboBoxProject_activated(int)));
	connect(comboBoxView, SIGNAL(activated(int)), this, SLOT(comboBoxView_activated(int)));

	connect(buttonCancelEdition, SIGNAL(clicked()), this, SLOT(buttonCancelEdition_clicked()));
	connect(buttonSaveFile, SIGNAL(clicked()), this, SLOT(buttonSaveFile_clicked()));
	connect(buttonLog, SIGNAL(clicked()), this, SLOT(buttonLog_clicked()));
	//connect(buttonBuild, SIGNAL(clicked()), this, SLOT(buttonBuild_clicked()));
	connect(buttonRemoveFile, SIGNAL(clicked()), this, SLOT(buttonRemoveFile_clicked()));
	connect(buttonAddFile, SIGNAL(clicked()), this, SLOT(buttonAddFile_clicked()));
	connect(buttonRemovePackage, SIGNAL(clicked()), this, SLOT(buttonRemovePackage_clicked()));
	connect(buttonLinkPackage, SIGNAL(clicked()), this, SLOT(buttonLinkPackage_clicked()));
	connect(buttonAddPackage, SIGNAL(clicked()), this, SLOT(buttonAddPackage_clicked()));
	connect(buttonAddProject, SIGNAL(clicked()), this, SLOT(buttonAddProject_clicked()));
	connect(buttonHome, SIGNAL(clicked()), this, SLOT(buttonHome_clicked()));
	connect(buttonConnect, SIGNAL(clicked()), this, SLOT(buttonConnect_clicked()));
	connect(buttonEditInfo, SIGNAL(clicked()), this, SLOT(buttonEditInfo_clicked()));
	connect(buttonUnWatch, SIGNAL(clicked()), this, SLOT(buttonUnWatch_clicked()));
	connect(buttonWatch, SIGNAL(clicked()), this, SLOT(buttonWatch_clicked()));
	connect(buttonRemoveProject, SIGNAL(clicked()), this, SLOT(buttonRemoveProject_clicked()));
	connect(buttonAddUserProject, SIGNAL(clicked()), this, SLOT(buttonAddUserProject_clicked()));
	connect(buttonRemoveUserProject, SIGNAL(clicked()), this, SLOT(buttonRemoveUserProject_clicked()));
	connect(buttonAddUserPackage, SIGNAL(clicked()), this, SLOT(buttonAddUserPackage_clicked()));
	connect(buttonRemoveUserPackage, SIGNAL(clicked()), this, SLOT(buttonRemoveUserPackage_clicked()));
	connect(buttonAddPlatform, SIGNAL(clicked()), this, SLOT(buttonAddPlatform_clicked()));
	connect(buttonRemovePlatform, SIGNAL(clicked()), this, SLOT(buttonRemovePlatform_clicked()));

	connect(kListViewFiles, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewFiles_selectionChanged(QListViewItem*)));
	connect(kListViewFiles, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(kListViewFiles_doubleClicked(QListViewItem*)));
	connect(kListViewPackage, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewPackage_selectionChanged(QListViewItem*)));
	connect(kListViewUser_project, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewUserPackage_selectionChanged(QListViewItem*)));
	connect(kListViewUser_package, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewUserProject_selectionChanged(QListViewItem*)));
	connect(kListViewProject, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewProject_selectionChanged(QListViewItem*)));
	connect(kListViewProject, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(kListViewProject_doubleClicked(QListViewItem*)));
	connect(kListViewWatched, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(kListViewWatched_selectionChanged(QListViewItem*)));
	connect(kListViewWatched, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(kListViewWatched_doubleClicked(QListViewItem*)));

	connect(stateMngr, SIGNAL(stateNone()), this, SLOT(stateNone()));
	connect(stateMngr, SIGNAL(stateConnected()), this, SLOT(stateConnected()));
	connect(stateMngr, SIGNAL(loadUserData()), this, SLOT(loadUserData()));
	connect(stateMngr, SIGNAL(loadAboutData()), this, SLOT(loadAboutData()));
	connect(stateMngr, SIGNAL(loadProjectMeta()), this, SLOT(loadProjectMeta()));
	connect(stateMngr, SIGNAL(loadProjectResult()), this, SLOT(loadProjectResult()));
	connect(stateMngr, SIGNAL(loadPackageMeta()), this, SLOT(loadPackageMeta()));
	connect(stateMngr, SIGNAL(loadPackageResult()), this, SLOT(loadPackageResult()));
	connect(stateMngr, SIGNAL(loadProjectListing()), this, SLOT(loadProjectListing()));
	connect(stateMngr, SIGNAL(loadPackageListing()), this, SLOT(loadPackageListing()));
	connect(stateMngr, SIGNAL(loadFileListing()), this, SLOT(loadFileListing()));
}

void IMainDlg::setupTabs() {
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(1), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(2), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(3), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(4), false);
}

void IMainDlg::setupActions(KActionCollection *actionCollection) {
	new KAction(i18n("About API"), 0, this, SLOT(helpAboutApi_activated()), actionCollection, "about_api");
	//new KAction(i18n("Preference"), 1, this, SLOT(preference_activated()), actionCollection, "preference");
}

/**********************************************/
void IMainDlg::buttonConnect_clicked() {
	if(isConnected)
		stateMngr->setState(st_none);
	else
		stateMngr->setState(st_connect);
}

void IMainDlg::buttonHome_clicked() {
	// switch to project view on home: project
	//TODO need to test if user home dir already exist if not send dialog to create it
	QString home = "home:";
	home.append(workset->getUserName());
	workset->setProject(home);
	stateMngr->setState(st_project_dirty);
	tabWidgetMain->showPage(tabWidgetMain->page(1));
}

void IMainDlg::kListViewProject_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();

	if ((item->text(0)) != (workset->getProject())) {
		workset->setProject(item->text(0));
		workset->setPackage("");
		workset->setProjectUser("");
		workset->setPackageUser("");
		workset->setPlatform("");
		workset->setFile("");
		workset->setArch("");
		workset->setRpm("");
	}
	if (!workset->getProject().isEmpty()) {
		kListViewFiles->clear();
		kListViewUser_project->clear();
		stateMngr->setState(st_project_dirty);
	}
	widgetStackProject->raiseWidget(0);
}


void IMainDlg::kListViewPackage_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();

	if ((item->text(0)) != (workset->getPackage())) {
		workset->setPackage(item->text(0));
		workset->setPackageUser("");
		workset->setFile("");
		workset->setArch("");
		workset->setRpm("");
		stateMngr->setState(st_package_dirty);
	}
	widgetStackProject->raiseWidget(3);
}

void IMainDlg::kListViewWatched_doubleClicked(QListViewItem* item) {
	workset->setProject(item->text(0));
	stateMngr->setState(st_project_dirty);
	tabWidgetMain->showPage(tabWidgetMain->page(1));
}

void IMainDlg::buttonEditInfo_clicked() {
	//TODO those belong to the preference dialog
}

/**********************************************/
void IMainDlg::buttonWatch_clicked() {
	//FIXME
	//workerMisc->toggleWatched();
}

void IMainDlg::buttonUnWatch_clicked() {
	//FIXME
	//workerMisc->toggleWatched();
}

void IMainDlg::kListViewWatched_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();
	workset->setProject(item->text(0));
	//stateMngr->setState(st_project_dirty);
	//NOTE if i do this i need todo some more cleanup on tab change
	//NOTE Should i set the project as dirty and reload all meta in background?
}

/**********************************************/
void IMainDlg::buttonAddUserProject_clicked() {
	stateMngr->request(rq_add_user);//FIXME
	stateMngr->setState(st_project_dirty);
}

void IMainDlg::buttonRemoveUserProject_clicked() {
	//NOTE aint possible with api atm anyway
	//workerProject->removeUser();
	//stateMngr->setState(st_project_dirty);
}

void IMainDlg::kListViewUserProject_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();
	workset->setProjectUser(item->text(0));
}

/**********************************************/
void IMainDlg::buttonAddUserPackage_clicked() {
	stateMngr->request(rq_add_user);//FIXME
	stateMngr->setState(st_package_dirty);
}

void IMainDlg::buttonRemoveUserPackage_clicked() {
	//TODO
	//workerPackage->removeUser();
	//stateMngr->setState(st_package_dirty);
}

void IMainDlg::kListViewUserPackage_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();
	workset->setPackageUser(item->text(0));
}

/**********************************************/
void IMainDlg::buttonAddProject_clicked() {
	stateMngr->request(rq_add_project);
}

void IMainDlg::buttonRemoveProject_clicked() {
	//TODO FIXME need to select another one too
	//workerProject->remove();
	//stateMngr->setState(st_project_dirty);
}

void IMainDlg::comboBoxProject_activated(int selection) {
	//FIXME
	//workerProject->setFilter(selection);
}

void IMainDlg::kListViewProject_doubleClicked(QListViewItem* item) {
	//FIXME select a package and emit a kListViewPackage_selectionChanged
	workset->setProject(item->text(0));
	stateMngr->setState(st_package_dirty);
}

/**********************************************/
void IMainDlg::buttonAddPlatform_clicked() {
	stateMngr->request(rq_add_platform);
}

void IMainDlg::buttonRemovePlatform_clicked() {
	//TODO
	//workerProject->removePlatform();
	//stateMngr->setState(st_platform_dirty);
}


/**********************************************/
void IMainDlg::buttonAddPackage_clicked() {
	stateMngr->request(rq_add_package);
}

void IMainDlg::buttonLinkPackage_clicked() {
	//TODO
	//workerPackage->addLink();
}

void IMainDlg::buttonRemovePackage_clicked() {
	//FIXME need to select another one too
	//workerPackage->remove();
	//stateMngr->setState(st_package_dirty);
}

void IMainDlg::comboBoxPackage_activated(int selection) {
	//FIXME
	//workerPackage->setFilter(selection);
}

/**********************************************/
void IMainDlg::buttonAddFile_clicked() {
	stateMngr->request(rq_add_file);
}

void IMainDlg::buttonRemoveFile_clicked() {
	//TODO
	//workerPackage->removeFile();
}

void IMainDlg::kListViewFiles_doubleClicked(QListViewItem* item) {
	//FIXME move this to state manager
	//workerPackage->getFile();
}

void IMainDlg::kListViewFiles_selectionChanged(QListViewItem* item) {
	if (!item) {
		return;}
	item->setSelected(true);
	item->repaint();
	workset->setFile(item->text(0));
}

/**********************************************/
void IMainDlg::buttonBuild_clicked() {
	//TODO
	//workerPackage->build();
}

void IMainDlg::buttonLog_clicked() {
	//TODO
	//workerPackage->getLog();
}

/**********************************************/
void IMainDlg::helpAboutApi_activated() {
	//FIXME move this to state manager
	//workerMisc->getAboutApi();
}

/**********************************************/
void IMainDlg::buttonSaveFile_clicked() {
	//TODO
}

void IMainDlg::buttonCancelEdition_clicked() {
	//TODO
}

void IMainDlg::kURLRequester_openFileDialog(KURLRequester*) {
	//TODO
}

void IMainDlg::kURLRequester_returnPressed(const QString&) {
	//TODO
}

/**********************************************/
void IMainDlg::comboBoxView_activated(int selection) {
	//FIXME switch amongst the stacked package view, dont forget dirty state
	switch (selection) {
	case 0: {
		//widgetStackProject->raiseWidget(0);
		break;}

	case 1: {
		//widgetStackProject->raiseWidget(1);
		break;}

	case 2: {
		//widgetStackProject->raiseWidget(2);
		break;}

	case 3: {
		//widgetStackProject->raiseWidget(3);
		break;}
	}
}

void IMainDlg::tabWidgetMain_currentChanged(QWidget* page) {
	int tab = tabWidgetMain->currentPageIndex();
	switch (tab) {
	case 1: {
		if (!kListViewProject->itemAtIndex(0)) {
			stateMngr->request(rq_project_listing);}
		if (!workset->getProject().isEmpty()) {
			stateMngr->setState(st_project_dirty);
			if (!workset->getPackage().isEmpty()) {
				stateMngr->setState(st_package_dirty);
			} else {
				kListViewFiles->clear();
				kListViewUser_package->clear();
			}
		}
		break;
	}

	case 2: {
		stateMngr->setState(st_edit);
		break;
	}

	case 0:break;
	case 3:break;
	case 4:break;

	default: kdDebug() << "undefined tab reached" << endl;
	
	}
}

void IMainDlg::changeUser(const QString& user) {
	workset->setUserName(user);
	if (!user.isEmpty()) {
		buttonConnect->setEnabled(true);
		connect(kLineEditUserName, SIGNAL(returnPressed()), this, SLOT(buttonConnect_clicked()));
	} else {
		buttonConnect->setEnabled(false);
		disconnect(kLineEditUserName, SIGNAL(returnPressed()), this, SLOT(buttonConnect_clicked()));
	}
}

void IMainDlg::stateConnected() {
	isConnected = true;
	buttonConnect->setText(tr("Disconnect"));
	buttonConnect->setEnabled(true);

	buttonEditInfo->setEnabled(true);
	buttonUnWatch->setEnabled(true);
	buttonHome->setEnabled(true);

	kListViewWatched->setEnabled(true);

	kLineEditUserName->setEnabled(false);
	kLineEditRealName->setEnabled(true);
	kLineEditEmail->setEnabled(true);
	kLineEditBackendRpm->setEnabled(true);
	kLineEditBackendSource->setEnabled(true);

	tabWidgetMain->setTabEnabled(tabWidgetMain->page(1), true);
	//tabWidgetMain_currentChanged(tabWidgetMain);
}

void IMainDlg::stateNone() {
	isConnected = false;
	buttonConnect->setText(tr("Connect"));
	buttonConnect->setEnabled(true);

	buttonEditInfo->setEnabled(false);
	buttonUnWatch->setEnabled(false);
	buttonHome->setEnabled(false);

	kListViewWatched->setEnabled(false);

	kLineEditUserName->setEnabled(true);
	kLineEditRealName->setEnabled(false);
	kLineEditEmail->setEnabled(false);
	kLineEditBackendRpm->setEnabled(false);
	kLineEditBackendSource->setEnabled(false);

	tabWidgetMain->page(0);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(1), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(2), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(3), false);
	tabWidgetMain->setTabEnabled(tabWidgetMain->page(4), false);
	//later gonna fix so one can use local stuff without being connected
}

/*Worker Signals**************************************/
void IMainDlg::loadUserData() {
	kdDebug() << "loading userdata" << endl;
	UserData* userData = workset->getUserData();

	kLineEditRealName->setText(userData->getRealName());
	kLineEditEmail->setText(userData->getEmail());

	Backend rBackend = userData->getRpmBackend();
	if (!rBackend.host.isEmpty()) {
		QString nRpmBackend = rBackend.host;
		nRpmBackend += ":";
		nRpmBackend += rBackend.port;
		kLineEditBackendRpm->setText(nRpmBackend);
	}

	Backend sBackend = userData->getSourceBackend();
	if (!sBackend.host.isEmpty()) {
		QString nSourceBackend = sBackend.host;
		nSourceBackend += ":";
		nSourceBackend += sBackend.port;
		kLineEditBackendSource->setText(nSourceBackend);
	}

	QStringList watched = userData->getWatchList();
	kListViewWatched->clear();
	if (watched.count() > 0) {
		watched.sort();
		for (QStringList::Iterator it = watched.begin(); it != watched.end(); ++it) {
			new KListViewItem(kListViewWatched, *it);
		}
	}

	kdDebug() << "userdata has been loaded" << endl;
}

/*
void IMainDlg::loadSetUserDataReply(Reply& data) {
	kdDebug() << "should load the answer to trying to set/write user data here" << endl;

}*/

void IMainDlg::loadAboutData() {
	kdDebug() << "loading about data" << endl;
	AboutData* aboutData = workset->getAboutData();
	dialog::AboutDialog* aboutDlg = new dialog::AboutDialog(this, "About Dialog", aboutData);

	aboutDlg->show();

	kdDebug() << "about data has been loaded" << endl;
}

void IMainDlg::loadProjectMeta() {
	kdDebug() << "loading project meta" << endl;
	ProjectMeta* projectMeta = workset->getProjectMeta();

	QString nTitle = tr("Title: ");
	nTitle += projectMeta->getTitle();
	kTextEditProjectTitle->setText(nTitle);

	QString nDescription = tr("Description: ");
	nDescription += projectMeta->getDescription();
	kTextEditProjectDescription->setText(nDescription);

	Users* pUsers = projectMeta->getUsers();
	kListViewUser_project->clear();
	if (pUsers->userId.count() > 0) {
		for (int Z = 0; Z != pUsers->userId.count(); ++Z) {
			new KListViewItem(kListViewUser_project, pUsers->userId[Z], pUsers->userRole[Z]);
		}
	}

	if (!workset->getProject().isEmpty()) {
		tabWidgetMain->setTabEnabled(tabWidgetMain->page(2), true);
	}

	checkOwnerProject();
	kdDebug() << "project meta has been loaded" << endl;
}

/*
void IMainDlg::loadProjectMetaReply(Reply& data) {
	kdDebug() << "should load the project reply here" << endl;
}*/

void IMainDlg::loadProjectListing() {
	kdDebug() << "loading project listing" << endl;
	Directory* projectListing = workset->getProjectListing();

	QStringList listing = projectListing->getListing();
	kListViewProject->clear();
	if (listing.count() > 0) {
		listing.sort();
		for (QStringList::Iterator it = listing.begin(); it != listing.end(); ++it) {
			new KListViewItem(kListViewProject, *it);
		}
	}

	if (!workset->getProject().isEmpty()) {
		//set view to current selected project
		QListViewItem* item = kListViewProject->findItem(workset->getProject(), 0, 0);
		kListViewProject->setCurrentItem(item);
		kListViewProject->ensureItemVisible(item);
		item->repaint();
	}

	kdDebug() << "project listing has been loaded" << endl;
}

void IMainDlg::loadPackageListing() {
	kdDebug() << "loading package listing" << endl;
	Directory* packageListing = workset->projectMeta.getPackageListing();

	QStringList listing = packageListing->getListing();
	kListViewPackage->clear();
	if (listing.count() > 0) {
		listing.sort();
		for (QStringList::Iterator it = listing.begin(); it != listing.end(); ++it) {
			new KListViewItem(kListViewPackage, *it);
		}
	}

	if (!workset->getPackage().isEmpty()) {
		//set view to current selected package
		QListViewItem* item = kListViewPackage->findItem(workset->getPackage(), 0, 0);
		kListViewPackage->setCurrentItem(item);
		kListViewPackage->ensureItemVisible(item);
		item->repaint();
	}

	kdDebug() << "package listing has been loaded" << endl;
}

void IMainDlg::loadFileListing() {
	kdDebug() << "loading file listing" << endl;
	Directory* fileListing = workset->packageMeta.getFileListing();

	QStringList listing = fileListing->getListing();
	kListViewFiles->clear();
	if (listing.count() > 0) {
		listing.sort();
		for (QStringList::Iterator it = listing.begin(); it != listing.end(); ++it) {
			new KListViewItem(kListViewFiles, *it);
		}
	}

	if (!workset->getFile().isEmpty()) {
		//set view to current selected package
		QListViewItem* item = kListViewFiles->findItem(workset->getFile(), 0, 0);
		kListViewFiles->setCurrentItem(item);
		kListViewFiles->ensureItemVisible(item);
		item->repaint();
	}

	kdDebug() << "file listing has been loaded" << endl;
}

void IMainDlg::loadProjectResult() {
	kdDebug() << "should load the project result here" << endl;
	ProjectResult* projectResult = workset->projectMeta.getResult();

}

void IMainDlg::loadPackageMeta() {
	kdDebug() << "loading package meta" << endl;
	PackageMeta* packageMeta = workset->getPackageMeta();

	QString nTitle = tr("Title: ");
	nTitle += packageMeta->getTitle();
	kTextEditPackageTitle->setText(nTitle);

	QString nDescription = tr("Description: ");
	nDescription += packageMeta->getDescription();
	kTextEditPackageDescription->setText(nDescription);

	Users* pUsers = packageMeta->getUsers();
	kListViewUser_package->clear();
	if (pUsers->userId.count() > 0) {
		for (int Z = 0; Z != pUsers->userId.count(); ++Z) {
			new KListViewItem(kListViewUser_package, pUsers->userId[Z], pUsers->userRole[Z]);
		}
	}

	checkOwnerPackage();
	kdDebug() << "package meta has been loaded" << endl;
}

/*
void IMainDlg::loadPackageMetaReply(Reply& data) {
	kdDebug() << "should load the reply to a set/write request of the package meta here" << endl;
	kdDebug() << "200 if everything went fine" << endl;

	stateMngr->setState(st_package_dirty);
	//workerProject->getPackageListing();
	//FIXME wont reload newly created package meta
	//workerPackage->getMeta();

}*/

/*
void IMainDlg::loadPackageAddFileReply(Reply& data) {
	kdDebug() << "should load the reply of a package add file request here" << endl;

}*/

/*
void IMainDlg::loadPackageAddSpecReply(Reply& data) {
	kdDebug() << "should load the reply of a package add spec request here" << endl;

}*/

void IMainDlg::loadPackageResult() {
	kdDebug() << "should load the package result here" << endl;
	PackageResult* packageResult = workset->packageMeta.getResult();

}

void IMainDlg::loadPackageLog(QString& data) {
	kdDebug() << "should load the log file here" << endl;

}

void IMainDlg::loadPlatformData() {

}

void IMainDlg::checkOwnerProject() {
	isProjectMainteneer = false;

	Users* pUsers = workset->getProjectMeta()->getUsers();
	if (pUsers->userId.count() > 0) {
		for (int Z = 0; Z != pUsers->userId.count(); ++Z) {
			 if (pUsers->userId[Z] == workset->getUserName()) {
				isProjectMainteneer = true;
			}
		}
	}

	if (isProjectMainteneer) {
		buttonSaveProject->setEnabled(true);
		buttonBuildProject->setEnabled(true);
		buttonAddPackage->setEnabled(true);
		buttonLinkPackage->setEnabled(true);
		buttonAddUserProject->setEnabled(true);

		kTextEditProjectTitle->setReadOnly(false);
		kTextEditProjectDescription->setReadOnly(false);
	} else {
		buttonSaveProject->setEnabled(false);
		buttonBuildProject->setEnabled(false);
		buttonAddPackage->setEnabled(false);
		buttonRemovePackage->setEnabled(false);
		buttonLinkPackage->setEnabled(false);
		buttonAddUserProject->setEnabled(false);
		buttonRemoveUserProject->setEnabled(false);

		kTextEditProjectTitle->setReadOnly(true);
		kTextEditProjectDescription->setReadOnly(true);
	}
}

void IMainDlg::checkOwnerPackage() {
	isPackageMainteneer = false;

	Users* pUsers = workset->getPackageMeta()->getUsers();
	if (pUsers->userId.count() > 0) {
		for (int Z = 0; Z != pUsers->userId.count(); ++Z) {
			 if (pUsers->userId[Z] == workset->getUserName()) {
				isPackageMainteneer = true;
			}
		}
	}

	if (isPackageMainteneer) {
		buttonAddFile->setEnabled(true);
		buttonBuildPackage->setEnabled(true);
		buttonSavePackage->setEnabled(true);
		buttonAddUserPackage->setEnabled(true);

		kTextEditPackageTitle->setReadOnly(false);
		kTextEditPackageDescription->setReadOnly(false);
	} else {
		buttonAddFile->setEnabled(false);
		buttonRemoveFile->setEnabled(false);
		buttonBuildPackage->setEnabled(false);
		buttonSavePackage->setEnabled(false);
		buttonAddUserPackage->setEnabled(false);
		buttonRemoveUserPackage->setEnabled(false);

		kTextEditPackageTitle->setReadOnly(true);
		kTextEditPackageDescription->setReadOnly(true);
	}
}
