// C++ Interface: iMainDlg
//
// Description: Main dialog interface for the buildservice richclient
//
// Note: ui need fixing to include a "sync" button in the project and package view
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef IMAINDLG_H
#define IMAINDLG_H

#include <qlabel.h>
#include <qobject.h>
#include <qlayout.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qcombobox.h>
#include <qtabwidget.h>
#include <qpushbutton.h>

#include <kdebug.h>
#include <klistview.h>
#include <klineedit.h>
#include <ktextedit.h>
#include <kmainwindow.h>

#include "package.h"
#include "project.h"
#include "miscdata.h"
#include "userdata.h"
#include "repository.h"

#include "workset.h"
#include "statemanager.h"
#include "mainview_base.h"

using namespace nbs;

class IMainDlg : public MainViewBase {
Q_OBJECT
public:
	IMainDlg(QWidget *parent , const char *name);
	~IMainDlg();

	void setupActions(KActionCollection*);

// ui slots
public slots:
	void buttonConnect_clicked();
	void buttonHome_clicked();
	void buttonEditInfo_clicked();
	void buttonWatch_clicked();
	void buttonUnWatch_clicked();
	void buttonAddUserProject_clicked();
	void buttonRemoveUserProject_clicked();
	void buttonAddUserPackage_clicked();
	void buttonRemoveUserPackage_clicked();
	void buttonAddProject_clicked();
	void buttonRemoveProject_clicked();
	void buttonAddPlatform_clicked();
	void buttonRemovePlatform_clicked();
	void buttonAddPackage_clicked();
	void buttonLinkPackage_clicked();
	void buttonRemovePackage_clicked();
	void buttonAddFile_clicked();
	void buttonRemoveFile_clicked();
	void buttonBuild_clicked();
	void buttonLog_clicked();
	void buttonSaveFile_clicked();
	void buttonCancelEdition_clicked();

	void helpAboutApi_activated();

	void kURLRequester_openFileDialog(KURLRequester*);
	void kURLRequester_returnPressed(const QString&);

	void comboBoxProject_activated(int);
	void comboBoxPackage_activated(int);
	void comboBoxView_activated(int);

	//TODO move this to the statemanager
	void tabWidgetMain_currentChanged(QWidget*);

	void kListViewWatched_doubleClicked(QListViewItem*);
	void kListViewWatched_selectionChanged(QListViewItem*);
	void kListViewUserProject_selectionChanged(QListViewItem*);
	void kListViewUserPackage_selectionChanged(QListViewItem*);
	void kListViewProject_selectionChanged(QListViewItem*);
	void kListViewProject_doubleClicked(QListViewItem*);
	void kListViewPackage_selectionChanged(QListViewItem*);
	void kListViewFiles_doubleClicked(QListViewItem*);
	void kListViewFiles_selectionChanged(QListViewItem*);

// other slots
public slots:
	void stateNone();
	void stateConnected();

	//FIXME data will be queried and received,
	// by the statemanager that will store them in the workset
	void loadUserData();
	void loadAboutData();
	void loadPlatformData();
	void loadProjectMeta();
	void loadProjectResult();
	void loadPackageMeta();
	void loadPackageResult();
	void loadPackageLog(QString&);
	void loadProjectListing();
	void loadPackageListing();
	void loadFileListing();

	//void loadProjectMetaReply(nbs::Reply&);
	//void loadSetUserDataReply(nbs::Reply&);
	//void loadPackageMetaReply(nbs::Reply&);
	//void loadPackageAddFileReply(nbs::Reply&);
	//void loadPackageAddSpecReply(nbs::Reply&);

protected:
	void setupButtons();
	void setupConnections();
	void setupListViews();
	void setupTabs();

	void checkOwnerProject();
	void checkOwnerPackage();

	void testConnect();

protected slots:
	void changeUser(const QString&);

private:
	//some stuff here can probably moved to the state manager
	bool isConnected;
	bool isProjectMainteneer;
	bool isPackageMainteneer;

	MainWorker* workset;
	StateManager* stateMngr;

};
#endif
