/*
    This file is part of Richclient.

    Copyright (c) 2006-2007 SUSE LINUX Products GmbH

    Author: Cornelius Schumacher <cschum@suse.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "mainwindow.h"
#include "mainwindow.moc"

#include "imaindlg.h"
#include "iprefdlg.h"

#include <kglobal.h>
#include <klocale.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

MainWindow::MainWindow() : KMainWindow(0, "MainWindow") {
	mView = new IMainDlg(this, "IMainDlg");
	//pView = new IPrefDlg(this, "Preference Dialog");

	setCentralWidget(mView);
	setupActions();
	setupGUI();
	
	resize(480, 600);
	setAutoSaveSettings();

	kdDebug() << "new MainWindow" << endl;
}

MainWindow::~MainWindow() {
	delete mView;
	//delete pView;

	kdDebug() << "deleted MainWindow" << endl;
}

void MainWindow::setupActions() {
	KStdAction::quit(this, SLOT(close()), actionCollection());
	KStdAction::preferences(this, SLOT(preferences()), actionCollection());
	
	mView->setupActions(actionCollection());
}

void MainWindow::preferences() {
	//pView->show();
}

void MainWindow::optionsConfigureToolbars() {
	// use the standard toolbar editor
	saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
	KEditToolbar dlg(actionCollection());
	connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(newToolbarConfig()));
	dlg.exec();
}

void MainWindow::newToolbarConfig() {
	// this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
	// recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
	createGUI();
	applyMainWindowSettings( KGlobal::config(), autoSaveGroup() );
}

bool MainWindow::queryClose() {
	return true;
}
