// C++ Implementation: MiscWorker
//
// Description: miscanlenous fonction
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "miscworker.h"


using namespace nbs;

MiscWorker::MiscWorker(QObject* parent = 0, const char* name = "MiscWorker", MainWorker* nWorkset = 0,
		SlaveWorker* nSlave = 0) : QObject(parent, name) {
	workset = nWorkset;
	slave   = nSlave;

	parser  = new xml::XmlParser();
	writer  = new xml::XmlWriter();

	connect(this, SIGNAL(newUserData()), parent, SLOT(newUserData()));
	connect(this, SIGNAL(newAboutData()), parent, SLOT(newAboutData()));
	connect(this, SIGNAL(newSetUserDataReply(nbs::Reply&)), parent, SLOT(newSetUserDataReply(nbs::Reply&)));

	kdDebug() << "new MiscWorker" << endl;
}

MiscWorker::~MiscWorker() {
	delete parser;
	delete writer;

	kdDebug() << "deleted MiscWorker" << endl;
}

void MiscWorker::toggleWatched() {
	QString proj = workset->getProject();
	//find in list if there remove else add
	//if () {
	//user.delWatchListElement(proj);
	// } else {
	//user.addWatchListElement(proj);
}	//}

//////////////////////////////////////////////////////
// GET /person/<userid>
void MiscWorker::getUserInfo(const QString& name) {
	kdDebug() << "sending user info request" << endl;
	QString userid = "person/";
	userid.append(name);
	gu_slave = new SlaveWorker(this, "Slave Worker");
	gu_slave->slaveGet(workset->formURL(dt_host, userid));
	connect(gu_slave, SIGNAL(gotData(const QString&)), this, SLOT(getUserInfoResult(const QString&)));
}

void MiscWorker::getUserInfoResult(const QString& data) {
	kdDebug() << "user data reached worker" << endl;
	disconnect(gu_slave, SIGNAL(gotData(const QString&)), this, SLOT(getUserInfoResult(const QString&)));
	workset->setUserData(parser->parseUserData(data));
	emit newUserData();
	//delete gu_slave;
}

//////////////////////////////////////////////////////
// PUT /person/<userid>
void MiscWorker::setUserInfo(const QString& name) {
	kdDebug() << "sending set user info request" << endl;
	QString userid = "person/";
	userid.append(name);
	su_slave = new SlaveWorker(this, "Slave Worker");
	su_slave->slavePut((workset->formURL(dt_host, userid)), (writer->writeUserData(workset->getUserData())));
	connect(su_slave, SIGNAL(gotData(const QString&)), this, SLOT(setUserInfoResult(const QString&)));
}

void MiscWorker::setUserInfoResult(const QString& data) {
	kdDebug() << "user data set reply reached worker" << endl;
	disconnect(su_slave, SIGNAL(gotData(const QString&)), this, SLOT(setUserInfoResult(const QString&)));
	Reply reply = parser->parseReply(data);
	emit newSetUserDataReply(reply);
	//delete su_slave;
}

//////////////////////////////////////////////////////
// GET /about
void MiscWorker::getAboutApi() {
	kdDebug() << "sending api info request" << endl;
	ga_slave = new SlaveWorker(this, "Slave Worker");
	ga_slave->slaveGet(workset->formURL(dt_host, "about"));
	connect(ga_slave, SIGNAL(gotData(const QString&)), this, SLOT(getAboutApiResult(const QString&)));
}

void MiscWorker::getAboutApiResult(const QString& data) {
	kdDebug() << "api info data reached worker" << endl;
	disconnect(ga_slave, SIGNAL(gotData(const QString&)), this, SLOT(getAboutApiResult(const QString&)));
	workset->setAboutData(parser->parseAbout(data));
	emit newAboutData();
	//delete ga_slave;
}
