// C++ Implementation: PackageWorker
//
// Description: package worker implementation
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "packageworker.h"

using namespace nbs;

PackageWorker::PackageWorker(QObject* parent = 0, const char* name = 0, MainWorker* nWorkset = 0
		) : QObject(parent, name) {
	workset = nWorkset;

	parser  = new xml::XmlParser();
	writer  = new xml::XmlWriter();

	connect(this, SIGNAL(newFileListing()), parent, SLOT(newFileListing()));
	connect(this, SIGNAL(newPackageMeta()), parent, SLOT(newPackageMeta()));
	connect(this, SIGNAL(newPackageResult()), parent, SLOT(newPackageResult()));
	connect(this, SIGNAL(newPackageLog(QString&)), parent, SLOT(newPackageLog(QString&)));
	connect(this, SIGNAL(newPackageMetaReply(nbs::Reply&)), parent, SLOT(newPackageMetaReply(nbs::Reply&)));
	connect(this, SIGNAL(newPackageAddFileReply(nbs::Reply&)), parent, SLOT(newPackageAddFileReply(nbs::Reply&)));
	connect(this, SIGNAL(newPackageAddSpecReply(nbs::Reply&)), parent, SLOT(newPackageAddSpecReply(nbs::Reply&)));

	kdDebug() << "new PackageWorker" << endl;
}

PackageWorker::~PackageWorker() {
	delete parser;
	delete writer;

	kdDebug() << "Deleted PackageWorker" << endl;
}

//////////////////////////////////////////////////////
// GET /source/<project>/<package>
void PackageWorker::getFileListing() {
	gfl_slave = new SlaveWorker(this, "Slave Worker");
	gfl_slave->slaveGet(workset->formURL(dt_package, ""));
	connect(gfl_slave, SIGNAL(gotData(const QString&)), this, SLOT(getFileListingResult(const QString&)));
}

void PackageWorker::getFileListingResult(const QString& data) {
	disconnect(gfl_slave, SIGNAL(gotData(const QString&)), this, SLOT(getFileListingResult(const QString&)));
	Directory listing = parser->parseDirectory(data);
	workset->packageMeta.setFileListing(listing);
	emit newFileListing();
}

//////////////////////////////////////////////////////
//get the project metada
// GET /source/<project>/<package>/_meta
void PackageWorker::getMeta() {
	gm_slave = new SlaveWorker(this, "Slave Worker");
	gm_slave->slaveGet(workset->formURL(dt_package, "_meta"));
	connect(gm_slave, SIGNAL(gotData(const QString&)), this, SLOT(getMetaResult(const QString&)));
}

void PackageWorker::getMetaResult(const QString& data) {
	disconnect(gm_slave, SIGNAL(gotData(const QString&)), this, SLOT(getMetaResult(const QString&)));
	PackageMeta meta = parser->parsePackageMeta(data);
	workset->setPackageMeta(meta);
	emit newPackageMeta();
}

//////////////////////////////////////////////////////
// PUT /source/<project>/<package>/_meta
void PackageWorker::setMeta() {
	sm_slave = new SlaveWorker(this, "Slave Worker");
	sm_slave->slavePut((workset->formURL(dt_package, "_meta")), (writer->writePackageMeta(workset->getPackageMeta())));
	connect(sm_slave, SIGNAL(gotData(const QString&)), this, SLOT(setMetaResult(const QString&)));
}

void PackageWorker::setMetaResult(const QString& data) {
	disconnect(sm_slave, SIGNAL(gotData(const QString&)), this, SLOT(setMetaResult(const QString&)));
	Reply reply = parser->parseReply(data);
	emit newPackageMetaReply(reply);
}

//////////////////////////////////////////////////////
// add a new package
void PackageWorker::add(QString name, QString title, QString description) {
	workset->getPackageMeta()->setName(name);
	workset->getPackageMeta()->setTitle(title);
	workset->getPackageMeta()->setDescription(description);

	workset->getPackageMeta()->setProject(workset->getProject());

	setMeta();//FIXME
	//also add file and/or create spec
}

//////////////////////////////////////////////////////
// PUT /source/<project>/<package>/_meta
void PackageWorker::addUser(QString name, UserType utype) {
	setMeta();//FIXME
}

//////////////////////////////////////////////////////
// need to erase it from the meta
void PackageWorker::removeUser() {
	setMeta();//FIXME
}

//////////////////////////////////////////////////////
//upload a new file to current package
// PUT /source/<project>/<package>/<filename>
void PackageWorker::addFile(KURL filepath) {
	af_slave = new SlaveWorker(this, "Slave Worker");
	af_slave->slavePutFile((workset->formURL(dt_package)), filepath);//FIXME
	connect(af_slave, SIGNAL(gotData(const QString&)), this, SLOT(addFileResult(const QString&)));
}

void PackageWorker::addFileResult(const QString& data) {
	disconnect(af_slave, SIGNAL(gotData(const QString&)), this, SLOT(addFileResult(const QString&)));
	Reply reply = parser->parseReply(data);
	emit newPackageAddFileReply(reply);
}

//////////////////////////////////////////////////////
//link existing package to current project
void PackageWorker::addLink() {

}

//////////////////////////////////////////////////////
//create new spec from template
// POST /source/<project>/<package>?cmd=createSpecFileTemplate
void PackageWorker::addSpec() {
	po_slave = new SlaveWorker(this, "Slave Worker");
	po_slave->slavePost((workset->formURL(dt_package)), "?cmd=createSpecFileTemplate");
	connect(po_slave, SIGNAL(gotData(const QString&)), this, SLOT(adSpecResult(const QString&)));
}

void PackageWorker::addSpecResult(const QString& data) {
	disconnect(po_slave, SIGNAL(gotData(const QString&)), this, SLOT(addSpecResult(const QString&)));
	Reply reply = parser->parseReply(data);
	emit newPackageAddSpecReply(reply);
}

//////////////////////////////////////////////////////
//download rpm from server
// GET /rpm/<project>/<platform>/<package>/<arch>/<rpmname>
void PackageWorker::getRpm() {
	ug_slave->slaveGetFile(workset->formURL(dt_rpm));
}

//////////////////////////////////////////////////////
//get file from server
// GET /source/<project>/<package>/<filename>
void PackageWorker::getFile() {
	ug_slave = new SlaveWorker(this, "Slave Worker");
	ug_slave->slaveGetFile(workset->formURL(dt_file));
	connect(ug_slave, SIGNAL(gotData(const QString&)), this, SLOT(getFileResult(const QString&)));
}

void PackageWorker::getFileResult(const QString& data) {
	disconnect(ug_slave, SIGNAL(gotData(const QString&)), this, SLOT(getFileResult(const QString&)));
	//QFile file = data;
	//emit newFile(file);
}

//////////////////////////////////////////////////////
//show result of a package rebuild
//return a text file
// GET /result/<project>/<platform>/<package>/<arch>/log
void PackageWorker::getLog() {
	gl_slave = new SlaveWorker(this, "Slave Worker");
	gl_slave->slaveGet(workset->formURL(dt_arch, "log"));
	connect(gl_slave, SIGNAL(gotData(const QString&)), this, SLOT(getLogResult(const QString&)));
}

void PackageWorker::getLogResult(const QString& data) {
	disconnect(gl_slave, SIGNAL(gotData(const QString&)), this, SLOT(getLogResult(const QString&)));
	QString log = data;
	emit newPackageLog(log);
}

//////////////////////////////////////////////////////
//show the current build status of a package
// GET /result/<project>/<platform>/<package>/result
void PackageWorker::getResult() {
	gr_slave = new SlaveWorker(this, "Slave Worker");
	gr_slave->slaveGet(workset->formURL(dt_package, "result"));
	connect(gr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getPackageResult(const QString&)));
}

void PackageWorker::getPackageResult(const QString& data) {
	disconnect(gr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getPackageResult(const QString&)));
	PackageResult result = parser->parsePackageResult(data);
	workset->packageMeta.setResult(result);
	emit newPackageResult();
}

//////////////////////////////////////////////////////
//remove selected package
//need to put the meta with package removed in it
// will server remove the files ?
void PackageWorker::remove() {
	setMeta();//FIXME
}

//////////////////////////////////////////////////////
//remove selected file
// DELETE /source/<project>/<package>/<filename>
void PackageWorker::removeFile() {
	ug_slave->slaveDelete(workset->formURL(dt_file));
}

//////////////////////////////////////////////////////
//trigger package rebuild
// POST /source/<project>/<package>?cmd=rebuild
void PackageWorker::build() {
	ug_slave->slavePost((workset->formURL(dt_package)), "?cmd=rebuild");
}

//////////////////////////////////////////////////////
// POST /source/<project>/<package>?cmd=commit&rev=<revision>&comment=<comment>&user=<user>
void PackageWorker::commit() {
	//FIXME
	int revision;
	QString comment, user, post;
	post.append("?cmd=commit&rev=");
	post.append(revision);
	post.append("&comment=");
	post.append(comment);
	post.append("&user=");
	post.append(user);
	ug_slave->slavePost((workset->formURL(dt_package)), post);
}

void PackageWorker::setFilter() {

}

void PackageWorker::sync() {

}

//int PackageWorker::getTail();
