// C++ Interface: PackageWorker
//
// Description: package worker interface
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef PACKAGEW_H
#define PACKAGEW_H

#include <qstring.h>
#include <qobject.h>

#include <kdebug.h>

#include "package.h"
#include "workset.h"
#include "miscdata.h"
#include "slaveworker.h"
#include "xmlparser.h"

namespace nbs {

/*******************************************************/
/* PackageWorker class                                 */
/*******************************************************/
class PackageWorker : public QObject {
Q_OBJECT
public:
	PackageWorker(QObject*, const char*, MainWorker*);
	~PackageWorker();

	void getMeta();
	void getFileListing();

	// name, title, description
	void add(QString, QString, QString);
	// file to add and file type
	void addFile(KURL);
	void addUser(QString, UserType);
	void addLink(/*FIXME*/);
	void addSpec();

	void remove();
	void removeFile();
	void removeUser();

	void getResult();
	//void getUsers();
	void getFile();
	void getRpm();
	void getLog();
	//int getTail();

	void sync();

	void disableArch();

	void build();
	void commit();

	void setFilter();

public slots:
	void getFileListingResult(const QString&);
	void getFileResult(const QString&);
	void getMetaResult(const QString&);
	void setMetaResult(const QString&);
	void addFileResult(const QString&);
	void addSpecResult(const QString&);
	void getLogResult(const QString&);
	void getPackageResult(const QString&);

signals:
	//void newFile();
	void newFileListing();
	void newPackageMeta();
	void newPackageResult();
	void newPackageLog(QString&);
	void newPackageMetaReply(nbs::Reply&);
	void newPackageAddFileReply(nbs::Reply&);
	void newPackageAddSpecReply(nbs::Reply&);

private:
	void setMeta();

	MainWorker* workset;
	xml::XmlParser* parser;
	xml::XmlWriter* writer;

	//SlaveWorker* slave;

	//FIXME: well this is ugly
	SlaveWorker* gm_slave;
	SlaveWorker* sm_slave;
	SlaveWorker* af_slave;
	SlaveWorker* po_slave;
	SlaveWorker* gl_slave;
	SlaveWorker* gr_slave;
	SlaveWorker* gfl_slave;

	// ug your friend the unclean slave
	SlaveWorker* ug_slave;
};

}
#endif
