// C++ Implementation: ProjectWorker
//
// Description: project worker implementation
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "projectworker.h"

using namespace nbs;

ProjectWorker::ProjectWorker(QObject* parent = 0, const char* name = 0, MainWorker* nWorkset = 0
		) : QObject(parent, name) {
	workset = nWorkset;
	parser  = new xml::XmlParser();
	writer  = new xml::XmlWriter();

	connect(this, SIGNAL(newProjectMeta()), parent, SLOT(newProjectMeta()));
	connect(this, SIGNAL(newProjectListing()), parent, SLOT(newProjectListing()));
	connect(this, SIGNAL(newPackageListing()), parent, SLOT(newPackageListing()));
	connect(this, SIGNAL(newProjectResult()), parent, SLOT(newProjectResult()));
	connect(this, SIGNAL(newProjectMetaReply(nbs::Reply&)), parent, SLOT(newProjectMetaReply(nbs::Reply&)));

	kdDebug() << "new ProjectWorker" << endl;
}

ProjectWorker::~ProjectWorker() {
	delete parser;
	delete writer;

	kdDebug() << "deleted ProjectWorker" << endl;
}

//////////////////////////////////////////////////////
//update a project info,
//well this is already done during user add and new project
//so prolly gonna be protected and called directly by other changing the meta
// PUT /source/<project>/_meta
void ProjectWorker::setMeta() {
	sm_slave = new SlaveWorker(this, "Slave Worker");
	sm_slave->slavePut((workset->formURL(dt_project)), (writer->writeProjectMeta(workset->getProjectMeta())));
	connect(sm_slave, SIGNAL(gotData(const QString&)), this, SLOT(setMetaResult(const QString&)));
}

void ProjectWorker::setMetaResult(const QString& data) {
	disconnect(sm_slave, SIGNAL(gotData(const QString&)), this, SLOT(setMetaResult(const QString&)));
	Reply reply = parser->parseReply(data);
	emit newProjectMetaReply(reply);
}

//////////////////////////////////////////////////////
// GET /source/<project>/_meta
void ProjectWorker::getMeta() {
	gm_slave = new SlaveWorker(this, "Slave Worker");
	gm_slave->slaveGet(workset->formURL(dt_project, "_meta"));
	connect(gm_slave, SIGNAL(gotData(const QString&)), this, SLOT(getMetaResult(const QString&)));
}

void ProjectWorker::getMetaResult(const QString& data) {
	disconnect(gm_slave, SIGNAL(gotData(const QString&)), this, SLOT(getMetaResult(const QString&)));
	ProjectMeta meta = parser->parseProjectMeta(data);
	workset->setProjectMeta(meta);
	emit newProjectMeta();
}

//////////////////////////////////////////////////////
//add a new project
// PUT /source/<project>/_meta
void ProjectWorker::add(QString name, QString title, QString description) {
	setMeta();//FIXME
	/*
	QString url = pworker->formURL(dt_project);
	// deal with old object
	// create empty project meta object
	//	add the stuff
	QString xml;
	pslave->slavePutXml(url, xml);
	// select as new project
	*/
}

//////////////////////////////////////////////////////
// PUT /source/<project>/_meta
void ProjectWorker::addUser(QString name, UserType utype) {
	setMeta();//FIXME
}

void ProjectWorker::addPlatform(QString name, QString target) {

}

//////////////////////////////////////////////////////
//erase the project (not supported?)
// not sure about this one need to check it out
// all the file and stuff will server deal with them ?
// DELETE /source/<project>/_meta
void ProjectWorker::remove() {

}

//////////////////////////////////////////////////////
// need to erase it from the meta
void ProjectWorker::removeUser() {
	setMeta();//FIXME
}

//////////////////////////////////////////////////////
// in the project meta in reallity
// this class prolly gonna get fused with the ProjectWorker
void ProjectWorker::removePlatform() {

}

//////////////////////////////////////////////////////
// GET /source/
void ProjectWorker::getProjectListing() {
	gpc_slave = new SlaveWorker(this, "Slave Worker");
	gpc_slave->slaveGet(workset->formURL(dt_host, "source"));
	connect(gpc_slave, SIGNAL(gotData(const QString&)), this, SLOT(getProjectListingResult(const QString&)));
}

void ProjectWorker::getProjectListingResult(const QString& data) {
	disconnect(gpc_slave, SIGNAL(gotData(const QString&)), this, SLOT(getProjectListingResult(const QString&)));
	Directory listing = parser->parseDirectory(data);
	workset->setProjectListing(listing);
	emit newProjectListing();
}

//////////////////////////////////////////////////////
// GET /source/<project>/
void ProjectWorker::getPackageListing() {
	gpr_slave = new SlaveWorker(this, "Slave Worker");
	gpr_slave->slaveGet(workset->formURL(dt_project));
	connect(gpr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getPackageListingResult(const QString&)));
}

void ProjectWorker::getPackageListingResult(const QString& data) {
	disconnect(gpr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getPackageListingResult(const QString&)));
	Directory listing = parser->parseDirectory(data);
	workset->projectMeta.setPackageListing(listing);
	emit newPackageListing();
}

//////////////////////////////////////////////////////
// GET /result/<project>/<platform>/result
void ProjectWorker::getResult() {
	gr_slave = new SlaveWorker(this, "Slave Worker");
	gr_slave->slaveGet(workset->formURL(dt_platform, "result"));
	connect(gr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getProjectResult(const QString&)));
}

void ProjectWorker::getProjectResult(const QString& data) {
	disconnect(gr_slave, SIGNAL(gotData(const QString&)), this, SLOT(getProjectResult(const QString&)));
	ProjectResult result = parser->parseProjectResult(data);
	workset->projectMeta.setResult(result);
	emit newProjectResult();
}

// currently only all and watched
void ProjectWorker::setFilter() {

}

void ProjectWorker::sync() {

}
