// C++ Interface: ProjectWorker
//
// Description: project worker interface
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef PROJECTW_H
#define PROJECTW_H

#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>

#include <kdebug.h>

#include "workset.h"
#include "project.h"
#include "miscdata.h"
#include "repository.h"
#include "slaveworker.h"
#include "xmlparser.h"

namespace nbs {

/*******************************************************/
/* ProjectWorker class                                 */
/*******************************************************/
class ProjectWorker : public QObject {
Q_OBJECT
public:
	ProjectWorker(QObject*, const char*, MainWorker*);
	~ProjectWorker();

	void getProjectListing();
	void getPackageListing();

	void getResult();
	void setFilter();

	void add(QString, QString, QString);
	void addUser(QString, UserType);
	void addPlatform(QString, QString);

	void remove();
	void removeUser();
	void removePlatform();

	void sync();

	void getMeta();
	void setMeta();

public slots:
	void getMetaResult(const QString&);
	void setMetaResult(const QString&);
	void getProjectListingResult(const QString&);
	void getPackageListingResult(const QString&);
	void getProjectResult(const QString&);

signals:
	void newProjectMeta();
	void newProjectResult();
	void newProjectListing();
	void newPackageListing();
	void newProjectMetaReply(nbs::Reply&);

private:
	MainWorker* workset;
	xml::XmlParser* parser;
	xml::XmlWriter* writer;

	//FIXME: well this is ugly
	SlaveWorker* sm_slave;
	SlaveWorker* gm_slave;
	SlaveWorker* gpc_slave;
	SlaveWorker* gpr_slave;
	SlaveWorker* gr_slave;
};

}
#endif
