// C++ Implementation: SlaveWorker
//
// Description: implementation of the worker actually doing network transfer
// and calling kio_slave
//
// atm mostly only the get function is usable the rest need some fixing
// note that current implementation isnt safe at all if multiple transfer are to happen at same time
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "slaveworker.h"

#include "httpheader.h"

using namespace nbs;
using namespace KIO;

SlaveWorker::SlaveWorker(QObject* parent = 0, const char* name = "SlaveWorker") {

	// call fcn and stuff

	kdDebug() << "new SlaveWorker" << endl;
}

SlaveWorker::~SlaveWorker() {
	kdDebug() << "deleted SlaveWorker" << endl;
}

//////////////////////////////////////////////////////
void SlaveWorker::slaveGet(KURL url) {
	TransferJob* jobGet = get(url, true, false);
	jobGet->addMetaData("cache", "reload");
	connect(jobGet, SIGNAL(result(KIO::Job*)), this, SLOT(getResult(KIO::Job*)));
	connect(jobGet, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(getData(KIO::Job*, const QByteArray&)));
	kdDebug() << "get request to " << url.path().latin1() << ", sent" << endl;

	//if 2 transfer happen at same time am screwed doing this that way
	dData = new QString();
}

void SlaveWorker::getData(KIO::Job*, const QByteArray& data) {
	if (!data.isEmpty()) {
		QString nData = data;
		dData->append(nData);
		kdDebug() << "data chunk received: \n" << nData.latin1() << endl;
	} else {
		kdDebug() << "data transfer completed" << endl;
	}
}

void SlaveWorker::getResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received get answer" << endl;
		kdDebug() << "emiting data" << endl;
		emit gotData(dData->latin1());
		delete dData;
	}
}

//////////////////////////////////////////////////////
void SlaveWorker::slaveGetFile(KURL url) {
	TransferJob* jobGFile = get(url, false, false);
	connect(jobGFile, SIGNAL(result(KIO::Job*)), this, SLOT(getFileResult(KIO::Job*)));
	connect(jobGFile, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(getFileData(KIO::Job*, const QByteArray&)));
	kdDebug() << "get file request to " << url.path().latin1() << endl;

	//if 2 transfer happen at same time am screwed doing this that way
	dData = new QString();
}

void SlaveWorker::getFileResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received get answer" << endl;
		kdDebug() << "emiting data" << endl;
		emit gotData(dData->latin1());
		delete dData;
	}
}

void SlaveWorker::getFileData(KIO::Job*, const QByteArray& data) {
	if (!data.isEmpty()) {
		QString nData = data;
		dData->append(nData);
		kdDebug() << "data chunk received: \n" << nData.latin1() << endl;
	} else {
		kdDebug() << "data transfer completed" << endl;
	}
}

//////////////////////////////////////////////////////
void SlaveWorker::slavePut(KURL url, QString data) {
	//QString path = url.path
	//path.append("/")
	//path.append(data)
	//url.setpath(path)
	int permissions = -1; // No special permissions
	bool showProgress = true;

	kdDebug() << "PUT " << url << endl;
	kdDebug() << "Body:" << data << endl;

	mUploadData = data;

	TransferJob* jobPut = put(url, permissions, true, false, showProgress );
	connect(jobPut, SIGNAL(result(KIO::Job*)), this, SLOT(putResult(KIO::Job*)));
	connect(jobPut, SIGNAL(data(KIO::Job*, const QByteArray&)),SLOT(putData(KIO::Job*, const QByteArray&)));
	connect(jobPut, SIGNAL(dataReq( KIO::Job*, QByteArray&)),
		SLOT( slotDataReq(KIO::Job*, QByteArray&)));
	jobPut->addMetaData("PropagateHttpHeader", "true");
	jobPut->addMetaData("accept", "text/xml");

	//if 2 transfer happen at same time am screwed doing this that way
	dData = new QString();
}

void SlaveWorker::putResult(KIO::Job* slaveJob) {
	HttpHeader header(slaveJob);

	kdDebug() << "PUT result: " << header.returnCode() << " " << header.returnText() << endl;

	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received put answer" << endl;
		kdDebug() << "emiting data" << endl;
		QString data(dData->latin1());
		emit gotData(data);
		delete dData;
	}
}

//FIXME this fuction is really ugly really should use a QIOdevice and a stream
void SlaveWorker::putData(KIO::Job*, const QByteArray& data) {
	if (!data.isEmpty()) {
		QString nData = data;
		dData->append(nData);
		kdDebug() << "data chunk received: \n" << nData.latin1() << endl;
	} else {
		kdDebug() << "data transfer completed" << endl;
	}
}

void SlaveWorker::slotDataReq( KIO::Job *, QByteArray &data )
{
	if (!mUploadData.isEmpty()) {
		data = mUploadData.utf8();
		data.resize(data.size() -1);
		mUploadData = QString::null;
	}
}


//////////////////////////////////////////////////////
void SlaveWorker::slavePutFile(const KURL remoteUrl, const KURL localUrl) {
	CopyJob* jobPFile = copy(localUrl, remoteUrl, true);
	connect(jobPFile, SIGNAL(result(Job*)), this, SLOT(putFileResult(Job*)));
	connect(jobPFile, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(putFileData(KIO::Job*, const QByteArray&)));
	kdDebug() << "put file request to " << remoteUrl.path().latin1() << ", " << localUrl.path().latin1() << endl;

	//if 2 transfer happen at same time am screwed doing this that way
	dData = new QString();
}

void SlaveWorker::putFileResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received put file answer" << endl;
		delete dData;
	}
}

void SlaveWorker::putFileData(KIO::Job*, const QByteArray& data) {

}

//////////////////////////////////////////////////////
void SlaveWorker::slavePost(KURL url, const QString command) {
	const QMemArray<char> postData;
//	postData = command;
	TransferJob* jobPost = http_post(url, postData, false);
	connect(jobPost, SIGNAL(result(KIO::Job*)), this, SLOT(postResult(KIO::Job*)));
	connect(jobPost, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(postData(KIO::Job*, const QByteArray&)));
	kdDebug() << "post request to " << url.path().latin1() << endl;
}

void SlaveWorker::postResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received post answer" << endl;
	}
}

void SlaveWorker::postData(KIO::Job*, const QByteArray& data) {

}

//////////////////////////////////////////////////////
void SlaveWorker::slaveDelete(KURL url) {
	SimpleJob* jobDel = file_delete(url, false);
	connect(jobDel, SIGNAL(result(KIO::Job*)), this, SLOT(deleteResult(KIO::Job*)));
	connect(jobDel, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(deleteData(KIO::Job*, const QByteArray&)));
	kdDebug() << "delete request to " << url.path().latin1() << endl;
}

void SlaveWorker::deleteResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received delete answer" << endl;
	}
}

void SlaveWorker::deleteData(KIO::Job*, const QByteArray& data) {

}

//////////////////////////////////////////////////////
void SlaveWorker::slaveDeleteRecursive(KURL url) {
	SimpleJob* jobDelR = file_delete(url, false);
	connect(jobDelR, SIGNAL(result(KIO::Job*)), this, SLOT(deleteResult(KIO::Job*)));
	connect(jobDelR, SIGNAL(data(KIO::Job*, const QByteArray&)), this, SLOT(deleteRecursiveData(KIO::Job*, const QByteArray&)));
	kdDebug() << "recursive delete request to " << url.path().latin1() << endl;
}

void SlaveWorker::deleteRecursiveResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received recursive delete answer" << endl;
	}
}

void SlaveWorker::deleteRecursiveData(KIO::Job*, const QByteArray& data) {

}

//////////////////////////////////////////////////////
void SlaveWorker::syncToServer() {
	//need to go through the whole local dir
	// add look for added project and file
	// also look for modificated spec

	//send all that to server
	kdDebug() << "syncing to server" << ", unimplemented" << endl;
}

void SlaveWorker::syncToResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received sync back answer" << ", unimplemented" << endl;
	}
}

//////////////////////////////////////////////////////
void SlaveWorker::syncfromServer() {
	// download everything locally project, or package
	kdDebug() << "syncing from server" << ", unimplemented" << endl;
}

void SlaveWorker::syncFromResult(KIO::Job* slaveJob) {
	if (slaveJob->error()) {
		slaveJob->showErrorDialog();
	} else {
		kdDebug() << "received sync answer" << ", unimplemented" << endl;
	}
}
