// C++ Interface: SlaveWorker
//
// Description: main class to call kio and do all network operation
// get and send file to server the returned qdom are also parsed from here
// by calling the XmlParser then returned as parsed XmlObject
//
//
// some kind of rsync would be nice for the workdir
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef SLAVEW_H
#define SLAVEW_H

#include <qobject.h>
#include <qtextstream.h>

#include <kdebug.h>
#include <kio/job.h>
#include <kio/slaveinterface.h>

#include "xmlparser.h"

class QString;

namespace nbs {

/*
template <class T, class U>
void slaveTransfer(T, U) {

inline (*transferX) (...);
inline (*transferY) (...);

}
*/

// suite de pointer vers founction

/*******************************************************/
/* SlaveWorker class                                   */
/*******************************************************/
class SlaveWorker : public QObject {
Q_OBJECT
public:
	SlaveWorker(QObject*, const char*/*, fcn ptr*/);
	~SlaveWorker();

	void slaveGet(KURL);
	void slaveGetFile(KURL);

	void slavePut(KURL, QString);
	void slavePutFile(const KURL, const KURL);

	void slavePost(KURL, const QString);

	void slaveDelete(KURL);
	void slaveDeleteRecursive(KURL);

	void syncToServer();
	void syncfromServer();

private slots:
	void getResult(KIO::Job*);
	void getData(KIO::Job*, const QByteArray&);
	void getFileResult(KIO::Job*);
	void getFileData(KIO::Job*, const QByteArray&);

	void putResult(KIO::Job*);
	void putData(KIO::Job*, const QByteArray&);
        void slotDataReq( KIO::Job *, QByteArray &data );

	void putFileResult(KIO::Job*);
	void putFileData(KIO::Job*, const QByteArray&);

	void postResult(KIO::Job*);
	void postData(KIO::Job*, const QByteArray&);

	void deleteResult(KIO::Job*);
	void deleteData(KIO::Job*, const QByteArray&);
	void deleteRecursiveResult(KIO::Job*);
	void deleteRecursiveData(KIO::Job*, const QByteArray&);

	void syncToResult(KIO::Job*);
	void syncFromResult(KIO::Job*);

signals:
	void gotData(const QString&);

private:
	QString* dData;

	QString mUploadData;
};

}
#endif
