// C++ Implementation: StateManager
//
// Description: 
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "statemanager.h"
#include "dialog.h"

using namespace nbs;

StateManager::StateManager(QObject* parent = 0, const char* name = 0, MainWorker* nWorkset = 0) : QObject(parent, name) {
	workset = nWorkset;
	slave   = new SlaveWorker(this, "Slave Worker");
	workerMisc     = new MiscWorker    (this, "Misc Worker", workset, slave);
	workerProject  = new ProjectWorker (this, "Project Worker", workset);
	workerPackage  = new PackageWorker (this, "Package Worker", workset);
	kdDebug() << "new StateManager" << endl;
}

StateManager::~StateManager() {
	delete workerMisc;
	delete workerProject;
	delete workerPackage;
	delete slave;
	kdDebug() << "deleted StateManager" << endl;
}

void StateManager::setState(State newState) {
	kdDebug() << "setting application state to " << newState << endl;
	currentState = newState;
	testState();
}

State StateManager::getState() {
	return currentState;
}

void StateManager::testState() {
	switch (currentState) {
	case st_none: {
		//NOTE maybe some way to kill all kio in progress?
		emit stateNone();
		break;
	}

	case st_connect: {
		request(rq_info);
		break;
	}

	case st_connected: {
		emit stateConnected();
		//connection is done and userdata loaded
		break;
	}

	case st_project_dirty: {
		workerProject->getMeta();
		workerProject->getPackageListing();
		//workerProject->getUsers();
		//TODO reference count?
		break;
	}

	case st_project: {
		//project completly loaded
		break;
	}

	case st_package_dirty: {
		workerPackage->getMeta();
		workerPackage->getFileListing();
		//workerPackage->getUsers();
		break;
	}

	case st_package: {
		//package completly loaded
		break;
	}

	case st_platform_dirty: {
		
		break;
	}

	case st_platform: {
		
		break;
	}

	kdDebug() << "undefined state has been achived" << endl;
	default: return;
	}

}

void StateManager::request(Request requestType) {
	switch (requestType) {
	case rq_project_listing: {
		workerProject->getProjectListing();
		break;
	}

	case rq_info: {
		workerMisc->getUserInfo(workset->getUserName());
		break;
	}

	case rq_add_project: {
		dialog::AddProject dlg;
		if (dlg.exec() == QDialog::Rejected) break;
		workset->setProject(dlg.projectName());
		workerProject->add(dlg.projectName(), dlg.projectTitle(), dlg.projectDescription());
		setState(st_project_dirty);
		break;
	}

	case rq_add_package: {
		dialog::AddPackage dlg;
		if (dlg.exec() == QDialog::Rejected) break;
		workset->setPackage(dlg.packageName());
		workerPackage->add(dlg.packageName(), dlg.packageTitle(), dlg.packageDescription());
		setState(st_package_dirty);
		break;
	}

	case rq_add_platform: {
		dialog::AddPlatform dlg;
		if (dlg.exec() == QDialog::Rejected) break;
		workset->setPlatform(dlg.platformName());
		workerProject->addPlatform(dlg.platformName(), dlg.platformTarget());
		setState(st_platform_dirty);
		break;
	}

	case rq_add_user: {
		dialog::AddUser dlg;
		if (dlg.exec() == QDialog::Rejected) break;
		//workerProject->addUser(dlg.userID(), dlg.userType());
		//workerPackage->addUser(dlg.userID(), dlg.userType());
		break;
	}

	case rq_add_file: {
		//KFileDialog* dlg = new KFileDialog("", "", this, "File Selector", 0);
		//dlg->show();
		//workerPackage->addFile(filepath);
		break;
	}

	kdDebug() << "undefined request" << endl;
	default: return;
	}

}

///////////////////
void StateManager::newUserData() {
	emit loadUserData();
	setState(st_connected);
}

void StateManager::newAboutData() {

	emit loadAboutData();
}

void StateManager::newSetUserDataReply(nbs::Reply& reply) {

	//emit loadSetUserDataReply(reply);
}

void StateManager::newProjectMeta() {
	emit loadProjectMeta();
	setState(st_project);
}

void StateManager::newProjectResult() {

	emit loadProjectResult();
}

void StateManager::newProjectMetaReply(nbs::Reply& reply) {

	//emit loadProjectMetaReply(reply);
}

void StateManager::newPackageMeta() {
	emit loadPackageMeta();
	setState(st_package);
}

void StateManager::newPackageResult() {

	emit loadPackageResult();
}

void StateManager::newPackageLog(QString& log) {

	emit loadPackageLog(log);
}

void StateManager::newPackageMetaReply(nbs::Reply& reply) {

	//emit loadPackageMetaReply(reply);
}

void StateManager::newPackageAddFileReply(nbs::Reply& reply) {

	//emit loadPackageAddFileReply(reply);
}

void StateManager::newPackageAddSpecReply(nbs::Reply& reply) {

	//emit loadPackageAddSpecReply(reply);
}

void StateManager::newProjectListing() {

	emit loadProjectListing();
}

void StateManager::newPackageListing() {

	emit loadPackageListing();
}

void StateManager::newFileListing() {

	emit loadFileListing();
}
