// C++ Interface: StateManager
//
// Description: 
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef STATEMNGR_H
#define STATEMNGR_H

#include <qobject.h>
#include <kdebug.h>

#include "workset.h"
#include "slaveworker.h"
#include "miscworker.h"
#include "projectworker.h"
#include "packageworker.h"

//TODO add a counter to really know when something isnt dirty anymore
//FIXME get the userdata 2 time

namespace nbs {

typedef enum State {
	st_none = 0,
	st_connect,
	st_connected,
	st_project_dirty,
	st_project,
	st_package_dirty,
	st_package,
	st_platform_dirty,
	st_platform,
	st_edit
};

typedef enum Request {
	rq_project_listing = 0,
	rq_info,
	rq_add_project,
	rq_add_package,
	rq_add_platform,
	rq_add_user,
	rq_add_file
};

class StateManager : public QObject {
Q_OBJECT
public:
	StateManager(QObject*, const char*, MainWorker*);
	~StateManager();

	State getState();
	void testState();
	void setState(State);
	void request(Request);

public slots:
	void newUserData();
	void newProjectMeta();
	void newProjectResult();
	void newAboutData();
	void newPackageMeta();
	void newPackageResult();
	void newProjectListing();
	void newPackageListing();
	void newFileListing();
	void newPackageLog(QString&);
	void newSetUserDataReply(nbs::Reply&);
	void newProjectMetaReply(nbs::Reply&);
	void newPackageMetaReply(nbs::Reply&);
	void newPackageAddFileReply(nbs::Reply&);
	void newPackageAddSpecReply(nbs::Reply&);

signals:
	void stateNone();
	void stateConnected();

	void loadUserData();
	void loadAboutData();
	void loadProjectMeta();
	void loadProjectResult();
	void loadPackageMeta();
	void loadPackageResult();
	void loadPackageLog(QString&);
	void loadProjectListing();
	void loadPackageListing();
	void loadFileListing();

	//void loadSetUserDataReply(nbs::Reply&);
	//void loadProjectMetaReply(nbs::Reply&);
	//void loadPackageMetaReply(nbs::Reply&);
	//void loadPackageAddFileReply(nbs::Reply&);
	//void loadPackageAddSpecReply(nbs::Reply&);

private:
	State currentState;

	SlaveWorker* slave;//FIXME should not be needed here
	MainWorker* workset;
	MiscWorker* workerMisc;
	ProjectWorker* workerProject;
	PackageWorker* workerPackage;
};

}
#endif
