// C++ Implementation: Workset / MainWorker
//
// Description: implementation of the workset holder and its worker
//  principally used to contrust query url
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "workset.h"
#include "prefs.h"

using namespace nbs;

MainWorker::MainWorker() {
	kdDebug() << "new workset" << endl;
	this->reset();
	loadConfig();
	kdDebug() << "new workset created" << endl;
}

MainWorker::~MainWorker() {
	writeConfig();
	kdDebug() << "deleted workset" << endl;
}

KURL MainWorker::formURL(DataType type, const QString morePath) {
	KURL remoteUrl;
	remoteUrl.setProtocol(w.protocol);
	remoteUrl.setHost(w.server);
	//remoteUrl.setPort(w.port);

	QString path;
	const QString slash  = "/";
	const QString source = "source";

	switch (type) {
		case dt_connect: {
			remoteUrl.setUser(w.username);
			break;
		}

		case dt_host: {
			break;
		}

		case dt_base: {
			path.append(source);
			break;
		}

		case dt_project: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			break;
		}

		case dt_platform: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			path.append(slash);
			path.append(w.platform);
			break;
		}

		case dt_package: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			path.append(slash);
			path.append(w.package);
			break;
		}

		case dt_file: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			path.append(slash);
			path.append(w.package);
			path.append(slash);
			path.append(w.file);
			break;
		}

		case dt_arch: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			path.append(slash);
			path.append(w.package);
			path.append(slash);
			path.append(w.file);
			path.append(slash);
			path.append(w.arch);
			break;
		}

		case dt_rpm: {
			path.append(slash);
			path.append(source);
			path.append(slash);
			path.append(w.project);
			path.append(slash);
			path.append(w.package);
			path.append(slash);
			path.append(w.file);
			path.append(slash);
			path.append(w.arch);
			path.append(slash);
			path.append(w.rpm);
			break;
		}

		kdDebug() << "undefined data type requested as url" << endl;
		default: break;
	}

	if(morePath) {
		path.append(slash);
		path.append(morePath);
	}
	remoteUrl.setPath(path);

	QString debug = remoteUrl.protocol() + "://" + remoteUrl.host() + "/" + path;
	kdDebug() << "formed url, " << debug.latin1() << endl;

	return remoteUrl;
}

bool MainWorker::setPort(const unsigned short int& value) {
	if (isValidPort(value)) {
		kdDebug() << "setting workset port to " << value << endl;
		w.port = value;
		return true;
	}
	return false;
}

bool MainWorker::setProtocol(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset protocol to " << value.latin1() << endl;
		w.protocol = value;
		return true;
	}
	return false;
}

bool MainWorker::setServer(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset server to " << value.latin1() << endl;
		w.server = value;
		return true;
	}
	return false;
}

bool MainWorker::setUserName(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset username to " << value.latin1() << endl;
		w.username = value;
		return true;
	}
	return false;
}

bool MainWorker::setWorkDir(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset workdir to " << value.latin1() << endl;
		w.workdir = value;
		return true;
	}
	return false;
}

bool MainWorker::setProject(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset project to " << value.latin1() << endl;
		w.project = value;
		return true;
	}
	return false;
}

bool MainWorker::setPlatform(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset platform to " << value.latin1() << endl;
		w.platform = value;
		return true;
	}
	return false;
}

bool MainWorker::setPackage(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset package to " << value.latin1() << endl;
		w.package = value;
		return true;
	}
	return false;
}

bool MainWorker::setFile(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset file to " << value.latin1() << endl;
		w.file = value;
		return true;
	}
	return false;
}

bool MainWorker::setProjectUser(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset user to " << value.latin1() << endl;
		w.projectUser = value;
		return true;
	}
	return false;
}

bool MainWorker::setPackageUser(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset user to " << value.latin1() << endl;
		w.packageUser = value;
		return true;
	}
	return false;
}

bool MainWorker::setArch(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset arch to " << value.latin1() << endl;
		w.arch = value;
		return true;
	}
	return false;
}

bool MainWorker::setRpm(const QString& value) {
	if (isValid(value)) {
		kdDebug() << "setting workset rpm to " << value.latin1() << endl;
		w.rpm = value;
		return true;
	}
	return false;
}

void MainWorker::setUserData(UserData data) {
	kdDebug() << "Changing UserData" << endl;
	userData = data;
}

void MainWorker::setAboutData(AboutData data) {
	kdDebug() << "Changing AboutData" << endl;
	aboutData = data;
}

void MainWorker::setProjectListing(Directory data) {
	kdDebug() << "Changing ProjectListing" << endl;
	projectListing = data;
}

void MainWorker::setPackageMeta(PackageMeta data) {
	kdDebug() << "Changing PackageMeta, " << w.package.latin1() << endl;
	packageMeta = data;
}

void MainWorker::setProjectMeta(ProjectMeta data) {
	kdDebug() << "Changing ProjectMeta, " << w.project.latin1() << endl;
	projectMeta = data;
}

// function returning value of the workset
unsigned short int MainWorker::getPort(){ return w.port; }
QString MainWorker::getProtocol()	{ return w.protocol; }
QString MainWorker::getServer()		{ return w.server; }
QString MainWorker::getUserName()	{ return w.username; }
QString MainWorker::getWorkDir()	{ return w.workdir; }
QString MainWorker::getProject()	{ return w.project; }
QString MainWorker::getPlatform()	{ return w.platform; }
QString MainWorker::getPackage()	{ return w.package; }
QString MainWorker::getFile()		{ return w.file; }
QString MainWorker::getPackageUser()	{ return w.packageUser; }
QString MainWorker::getProjectUser()	{ return w.projectUser; }
QString MainWorker::getArch()		{ return w.arch; }
QString MainWorker::getRpm()		{ return w.rpm; }

UserData* MainWorker::getUserData() {
	return &userData;
}

AboutData* MainWorker::getAboutData() {
	return &aboutData;
}

Directory* MainWorker::getProjectListing() {
	return &projectListing;
}

PackageMeta* MainWorker::getPackageMeta() {
	return &packageMeta;
}

ProjectMeta* MainWorker::getProjectMeta() {
	return &projectMeta;
}

/////////////////////////////////////////////////////////////////////
// useless now ?
void MainWorker::reset () {
	w.port		= 80;
	w.protocol	= "";
	w.server	= "";
	w.username	= "";
	w.workdir	= "";
	w.project	= "";
	w.platform	= "";
	w.package	= "";
	w.file		= "";
	w.projectUser	= "";
	w.packageUser	= "";
	w.arch		= "";
	w.rpm		= "";
}

// name validity testing, currently not implemented
// cant contain space slash and colons in all name sent to server
bool MainWorker::isValid(const QString& value) {
	if (value)//FIXME
	return true;
	return false;
}

bool MainWorker::isValidPort(const unsigned short int& port) {
	if (port)//FIXME
	return true;
	return false;
}

void MainWorker::loadConfig() {
	setServer(Prefs::serverurl());
	setUserName(Prefs::username());
	setProtocol(Prefs::protocol());
	setPort(Prefs::port());

	kdDebug() << "done loading config" << endl;
}

void MainWorker::writeConfig() {
	Prefs::setServerurl(w.server);
	Prefs::setUsername(w.username);
	Prefs::setProtocol(w.protocol);
	Prefs::setPort(w.port);
	Prefs::writeConfig();

	kdDebug() << "wrote config" << endl;
}
