// C++ Interface: workset
//
// Description: Mainworker interface and workset
//  workset holding selected stuff and url
//  used to construct the request url
//
// FIXME xml object in the workset should be accessed by a map
//
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef WORKSET_H
#define WORKSET_H

#include <qdir.h>
#include <qfile.h>
#include <qstring.h>

#include <kurl.h>
#include <kdebug.h>

#include "miscdata.h"
#include "userdata.h"
#include "package.h"
#include "project.h"
#include "repository.h"

namespace nbs {

// url type used when constructing the kio call
typedef enum DataType {
	dt_connect = 0,
	dt_host,
	dt_base,
	dt_project,
	dt_platform,
	dt_package,
	dt_file,
	dt_arch,
	dt_rpm
};

struct WorkSet {
	//protocol wont be used for now anyway always http
	unsigned short int port;
	QString protocol, server;
	QString username;
	QString workdir;
	QString project, platform, package;
	QString file;
	QString projectUser, packageUser;
	QString arch;
	QString rpm;
};

/*******************************************************/
/* MainWorker class                                    */
/*******************************************************/
class MainWorker {
public:
	MainWorker();
	~MainWorker();

	KURL formURL(DataType, const QString = "");

	bool setPort(const unsigned short int&);
	bool setProtocol(const QString&);
	bool setServer(const QString&);

	bool setUserName(const QString&);
	bool setWorkDir(const QString&);

	bool setProject(const QString&);
	bool setPlatform(const QString&);
	bool setPackage(const QString&);
	bool setFile(const QString&);
	bool setProjectUser(const QString&);
	bool setPackageUser(const QString&);
	bool setArch(const QString&);
	bool setRpm(const QString&);

	void setUserData(UserData);
	void setAboutData(AboutData);
	void setProjectListing(Directory);

	void setPackageMeta(PackageMeta);
	void setProjectMeta(ProjectMeta);

	unsigned short int getPort();
	QString getProtocol();
	QString getServer();

	QString getUserName();
	QString getWorkDir();

	QString getProject();
	QString getPlatform();
	QString getPackage();
	QString getFile();
	QString getProjectUser();
	QString getPackageUser();
	QString getArch();
	QString getRpm();

	UserData* getUserData();
	AboutData* getAboutData();
	Directory* getProjectListing();

	PackageMeta* getPackageMeta();
	ProjectMeta* getProjectMeta();

	//errrr, those data should be private
	ProjectMeta projectMeta;
	PackageMeta packageMeta;

protected:
	void reset();

	bool isValid(const QString&);
	bool isValidPort(const unsigned short int&);

	void loadConfig();
	void writeConfig();

private:
	KURL localUrl; //for copying file and syncing to/from server
	QFile config;
	WorkSet w;

	UserData userData;
	AboutData aboutData;

	Directory projectListing;
};

}
#endif
