// C++ Implementation: XmlParser, XmlWriter
//
// Description: xml parsing class, also XmlWriter atm converting back object to xml
//
// When evaluating the xml type html is to be considered an error
// (eg: during connection html is cancelled or failled)
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#include "xmlparser.h"

using namespace nbs;
using namespace xml;

/*******************************************************/
/* XmlParser class                                     */
/*******************************************************/
XmlParser::XmlParser() {
	kdDebug() << "new XmlParser" << endl;
}

XmlParser::~XmlParser() {
	kdDebug() << "deleted XmlParser" << endl;
}

Directory XmlParser::parseDirectory(const QString& xml) {
	QDomDocument dirDom;
	dirDom.setContent(xml, false);
	QDomElement root = dirDom.documentElement();
	QDomNode node = root.firstChild();
	Directory obj;

	if (root.tagName() != "directory") {
		kdDebug() << "wrong XML type. expected directory, got " << root.tagName().latin1() << endl;
	} else {
		kdDebug() << "Count: " << root.toElement().attribute("count") << endl;
		obj.setCount(root.toElement().attribute("count").toInt());
		while (!node.isNull()) {
			if (node.toElement().tagName() == "entry") {
				kdDebug() << "  entry: " << node.toElement().attribute("name").latin1() << endl;
				obj.addEntry(node.toElement().attribute("name"));
			}
			node = node.nextSibling();
		}
		kdDebug() << "directory xml succesfully parsed" << endl;
	}
	return obj;
}

ProjectMeta XmlParser::parseProjectMeta(const QString& xml) {
	QDomDocument metaDom;
	metaDom.setContent(xml, false);
	QDomElement root = metaDom.documentElement();
	QDomNode node = root.firstChild();
	ProjectMeta obj;

	if (root.tagName() != "project") {
		kdDebug() << "wrong XML type. expected project, got " << root.tagName().latin1() << endl;
	} else {
		while (!node.isNull()) {
			if (node.toElement().tagName() == "title") {
				kdDebug() << "title: " << node.toElement().text().latin1() << endl;
				obj.setTitle(node.toElement().text());
			}
	
			if (node.toElement().tagName() == "description") {
				kdDebug() << "description: " << node.toElement().text().latin1() << endl;
				obj.setDescription(node.toElement().text());
			}
	
			if (node.toElement().tagName() == "person") {
				kdDebug() << "person" << endl;
				kdDebug() << "  role: " << node.toElement().attribute("role").latin1() << endl;
				kdDebug() << "  userid: " << node.toElement().attribute("userid").latin1() << endl;
				obj.addUser(node.toElement().attribute("role"), node.toElement().attribute("userid"));
			}
	
			if (node.toElement().tagName() == "repository") {
	
			}
			node = node.nextSibling();
		}
		kdDebug() << "project sucefully parsed" << endl;
	}
	return obj;
}

ProjectResult XmlParser::parseProjectResult(const QString& xml) {
	QDomDocument resultDom;
	resultDom.setContent(xml, false);
	QDomElement root = resultDom.documentElement();
	QDomNode node = root.firstChild();
	ProjectResult obj;

	if (root.tagName() != "projectresult") {
		kdDebug() << "wrong XML type. expected projectresult, got " << root.tagName().latin1() << endl;
	} else {
		// also need to parse the root element as it carry some info
		while (!node.isNull()) {
			if (node.toElement().tagName() == "date") {
	
			}
	
			if (node.toElement().tagName() == "status") {
	
			}
	
			if (node.toElement().tagName() == "repositoryresult") {
	
			}
			node = node.nextSibling();
		}
		kdDebug() << "projectresult sucesfully parsed" << endl;
	}
	return obj;
}

PackageMeta XmlParser::parsePackageMeta(const QString& xml) {
	QDomDocument metaDom;
	metaDom.setContent(xml, false);
	QDomElement root = metaDom.documentElement();
	QDomNode node = root.firstChild();
	PackageMeta obj;

	if (root.tagName() != "package") {
		kdDebug() << "wrong XML type. expected package, got " << root.tagName().latin1() << endl;
	} else {
		while (!node.isNull()) {
			if (node.toElement().tagName() == "title") {
				kdDebug() << "title: " << node.toElement().text().latin1() << endl;
				obj.setTitle(node.toElement().text());
			}

			if (node.toElement().tagName() == "description") {
				kdDebug() << "description: " << node.toElement().text().latin1() << endl;
				obj.setDescription(node.toElement().text());
			}

			if (node.toElement().tagName() == "person") {
				kdDebug() << "person" << endl;
				kdDebug() << "  role: " << node.toElement().attribute("role").latin1() << endl;
				kdDebug() << "  userid: " << node.toElement().attribute("userid").latin1() << endl;
				obj.addUser(node.toElement().attribute("role"), node.toElement().attribute("userid"));
			}

			if (node.toElement().tagName() == "group") {
				kdDebug() << "group: " << node.toElement().text().latin1() << endl;
				obj.setGroup(node.toElement().text());
			}
			node = node.nextSibling();
		}
		kdDebug() << "package sucesfully parsed" << endl;
	}
	return obj;
}

PackageResult XmlParser::parsePackageResult(const QString& xml) {
	QDomDocument resultDom;
	resultDom.setContent(xml, false);
	QDomElement root = resultDom.documentElement();
	QDomNode node = root.firstChild();
	PackageResult obj;

	if (root.tagName() != "packageresult") {
		kdDebug() << "wrong XML type. expected packageresult, got " << root.tagName().latin1() << endl;
	} else {
		// also need to parse the root element as it carry some info
		while (!node.isNull()) {
			if (node.toElement().tagName() == "date") {
	
			}
	
			if (node.toElement().tagName() == "status") {
	
			}
	
			if (node.toElement().tagName() == "archresult") {
	
			}
			node = node.nextSibling();
		}
		kdDebug() << "packageresult sucesfully parsed" << endl;
	}
	return obj;
}

Reply XmlParser::parseReply(const QString& xml) {
	QDomDocument replyDom;
	replyDom.setContent(xml, false);
	QDomElement root = replyDom.documentElement();
	QDomNode node = root.firstChild();
	Reply obj;

	if (root.tagName() != "status") {
		kdDebug() << "wrong XML type. expected server reply, got " << root.tagName().latin1() << endl;
	} else {
		kdDebug() << "code: " << root.toElement().attribute("code") << endl;
		obj.setCode(root.toElement().attribute("code"));
		while (!node.isNull()) {
			if (node.toElement().tagName() == "summary") {
				kdDebug() << "  summary: " << node.toElement().text().latin1() << endl;
				obj.setSummary(node.toElement().text());
			}
			if (node.toElement().tagName() == "details") {
				kdDebug() << "  details: " << node.toElement().text().latin1() << endl;
				obj.setDetails(node.toElement().text());
			}
			node = node.nextSibling();
		}
		kdDebug() << "reply sucesfully parsed" << endl;
	}
	return obj;
}

AboutData XmlParser::parseAbout(const QString& xml) {
	QDomDocument aboutDom;
	aboutDom.setContent(xml, false);
	QDomElement root = aboutDom.documentElement();
	QDomNode node = root.firstChild();
	AboutData obj;

	if (root.tagName() != "about") {
		kdDebug() << "wrong XML type. expected about, got " << root.tagName().latin1() << endl;
	} else {
		while (!node.isNull()) {
			if (node.toElement().tagName() == "title") {
				kdDebug() << "title: " << node.toElement().text().latin1() << endl;
				obj.setTitle(node.toElement().text());
			}
			if (node.toElement().tagName() == "description") {
				kdDebug() << "description: " << node.toElement().text().latin1() << endl;
				obj.setDescription(node.toElement().text());
			}
			if (node.toElement().tagName() == "revision") {
				kdDebug() << "revision: " << node.toElement().text().latin1() << endl;
				obj.setRevision(node.toElement().text());
			}
			if (node.toElement().tagName() == "documentation") {
				kdDebug() << "documentation: type: " << node.toElement().attribute("type").latin1() <<
					", url: " << node.toElement().attribute("url").latin1() << endl;
				obj.addDocumentation(node.toElement().attribute("type"),
					node.toElement().attribute("url"));
			}
			node = node.nextSibling();
		}
		kdDebug() << "about sucesfully parsed" << endl;
	}
	return obj;
}

UserData XmlParser::parseUserData(const QString& xml) {
	QDomDocument userDom;
	userDom.setContent(xml, false);
	QDomElement root = userDom.documentElement();
	QDomNode node = root.firstChild();
	UserData obj;

	if (root.tagName() != "person") {
		kdDebug() << "wrong XML type. expected person, got " << root.tagName().latin1() << endl;
	} else {
		while (!node.isNull()) {
			if (node.toElement().tagName() == "login") {
				kdDebug() << "login: " << node.toElement().text().latin1() << endl;
				obj.setUserName(node.toElement().text());
			}
			if (node.toElement().tagName() == "email") {
				kdDebug() << "email: " << node.toElement().text().latin1() << endl;
				obj.setEmail(node.toElement().text());
			}
			if (node.toElement().tagName() == "realname") {
				kdDebug() << "realname: " << node.toElement().text().latin1() << endl;
				obj.setRealName(node.toElement().text());
			}
			if (node.toElement().tagName() == "source_backend") {
				QDomNode srcNode = node.firstChild();
				kdDebug() << "source backend" << endl;
				while (!srcNode.isNull()) {
					if (srcNode.toElement().tagName() == "host") {
						kdDebug() << "  host: " << srcNode.toElement().text().latin1() << endl;
						obj.setSourceHost(srcNode.toElement().text());
					}
					if (srcNode.toElement().tagName() == "port") {
						kdDebug() << "  port: " << srcNode.toElement().text().latin1() << endl;
						obj.setSourcePort(srcNode.toElement().text());
					}
					srcNode = srcNode.nextSibling();
				}
			}
			if (node.toElement().tagName() == "rpm_backend") {
				QDomNode rpmNode = node.firstChild();
				kdDebug() << "rpm backend" << endl;
				while (!rpmNode.isNull()) {
					if (rpmNode.toElement().tagName() == "host") {
						kdDebug() << "  host: " << rpmNode.toElement().text().latin1() << endl;
						obj.setRpmHost(rpmNode.toElement().text());
					}
					if (rpmNode.toElement().tagName() == "port") {
						kdDebug() << "  port: " << rpmNode.toElement().text().latin1() << endl;
						obj.setRpmPort(rpmNode.toElement().text());
					}
					rpmNode = rpmNode.nextSibling();
				}
			}
			if (node.toElement().tagName() == "watchlist") {
				QDomNode watchNode = node.firstChild();
				kdDebug() << "watchlist" << endl;
				while (!watchNode.isNull()) {
					if (watchNode.toElement().tagName() == "project") {
						kdDebug() << "  project: " << watchNode.toElement().attribute("name").latin1() << endl;
						obj.addWatchListElement(watchNode.toElement().attribute("name"));
					}
					watchNode = watchNode.nextSibling();
				}
			}
			node = node.nextSibling();
		}
		kdDebug() << "person sucesfully parsed" << endl;
	}
	return obj;
}

/*******************************************************/
/* XmlWriter class                                     */
/*******************************************************/
XmlWriter::XmlWriter() {
	kdDebug() << "new XmlWriter" << endl;
}

XmlWriter::~XmlWriter() {
	kdDebug() << "deleted XmlWriter" << endl;
}

QString XmlWriter::writeProjectMeta(const ProjectMeta* project) {
	QDomDocument doc;

	QDomElement topElement = doc.createElement("project");
	doc.appendChild(topElement);

	topElement.setAttribute("name", project->getName());
	/*topElement.setAttribute("project", project->getProject());*/

	createChild(doc, topElement, "title", project->getTitle());
	forceCreateChild(doc, topElement, "description", project->getDescription());

	const QString xml = doc.toString();
	return xml;
}

QString XmlWriter::writePackageMeta(const PackageMeta* package) {
	QDomDocument doc;

	QDomElement topElement = doc.createElement("package");
	doc.appendChild(topElement);

	topElement.setAttribute("name", package->getName());
	topElement.setAttribute("project", package->getProject());

	createChild(doc, topElement, "title", package->getTitle());
	forceCreateChild(doc, topElement, "description", package->getDescription());

	const QString xml = doc.toString();
	return xml;
}

QString XmlWriter::writeUserData(const UserData* object) {
	QDomDocument doc;

	QDomElement topElement = doc.createElement("person");
	doc.appendChild(topElement);

	createChild(doc, topElement, "login", object->getUserName());
	createChild(doc, topElement, "email", object->getEmail());
	createChild(doc, topElement, "realname", object->getRealName());

	const QString xml = doc.toString();
	return xml;
}

QDomElement XmlWriter::createChild( QDomDocument& doc, QDomElement& element, const QString& tagName ) {
	QDomElement e = doc.createElement(tagName);
	element.appendChild(e);

	return e;
}

void XmlWriter::createChild(QDomDocument& doc, QDomElement& element,
		const QString& tagName, const QString& text) {
	if (text.isEmpty()) return;

	QDomElement e = doc.createElement(tagName);
	element.appendChild(e);

	QDomText t = doc.createTextNode(text);
	e.appendChild(t);

	return;
}

QDomElement XmlWriter::forceCreateChild(QDomDocument& doc, QDomElement& element,
		const QString& tagName, const QString& text) {
	QDomElement e = doc.createElement(tagName);
	element.appendChild(e);

	if (!text.isEmpty()) {
		QDomText t = doc.createTextNode(text);
		e.appendChild(t);
	}

	return e;
}
