// C++ Interface: xmlParser and xmlWriter
//
// Description: class doing xml parsing and writing
//
// Author: Jonathan Arsenault <jonharson@gmail.com>, (C) 2006-2007
//
// Copyright: See COPYING file that comes with this distribution
//

#ifndef XMLPARSER_H
#define XMLPARSER_H

#include <qdom.h>
#include <qstring.h>

#include <kdebug.h>

#include "miscdata.h"
#include "userdata.h"
#include "package.h"
#include "project.h"
#include "repository.h"

namespace nbs {
namespace xml {

/*******************************************************/
/* XmlParser class                                     */
/*******************************************************/
class XmlParser {
public:
	XmlParser();
	~XmlParser();

	Directory parseDirectory(const QString&);

	ProjectMeta parseProjectMeta(const QString&);
	PackageMeta parsePackageMeta(const QString&);

	ProjectResult parseProjectResult(const QString&);
	PackageResult parsePackageResult(const QString&);

	Reply parseReply(const QString&);
	UserData parseUserData(const QString&);
	AboutData parseAbout(const QString&);
};

/*******************************************************/
/* XmlWriter class                                     */
/*******************************************************/
class XmlWriter {
public:
	XmlWriter();
	~XmlWriter();

	QString writeUserData(const UserData*);
	QString writeProjectMeta(const ProjectMeta*);
	QString writePackageMeta(const PackageMeta*);

protected:
	QDomElement createChild(QDomDocument&, QDomElement&, const QString&);
	void createChild(QDomDocument&, QDomElement&, const QString&, const QString&);
	QDomElement forceCreateChild(QDomDocument&, QDomElement&, const QString&, const QString&);

};

}}
#endif
